from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from datetime import datetime
from typing import (
    Any,
    Dict,
    List,
    Literal,
    Union,
)
from uuid import UUID

AnyValue = Any

class ApiClientInformation(ObjectModel):
    identifier: str = ApiField(alias='identifier')
    type: str = ApiField(alias='type')

class ApplicationInformation(ObjectModel):
    certificateIssuingTemplateAliasIdMap: Dict[str, UUID] = ApiField(
        alias='certificateIssuingTemplateAliasIdMap',
        default_factory=dict
    )
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    description: str = ApiField(alias='description')
    fullyQualifiedDomainNames: List[str] = ApiField(alias='fullyQualifiedDomainNames', default_factory=list)
    id: UUID = ApiField(alias='id')
    ipRanges: List[str] = ApiField(alias='ipRanges', default_factory=list)
    modificationDate: datetime = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    ownerIdsAndTypes: List[OwnerIdAndType] = ApiField(alias='ownerIdsAndTypes', default_factory=list)
    ownership: ApplicationOwnership = ApiField(alias='ownership')
    owningServiceAccounts: List[ServiceAccountInformation] = ApiField(
        alias='owningServiceAccounts',
        default_factory=list
    )
    owningTeams: List[TeamInformation] = ApiField(alias='owningTeams', default_factory=list)
    owningUsers: List[UserInformation] = ApiField(alias='owningUsers', default_factory=list)
    ports: List[str] = ApiField(alias='ports', default_factory=list)

class ApplicationOwnership(ObjectModel):
    owningTeams: List[UUID] = ApiField(alias='owningTeams', default_factory=list)
    owningUsers: List[UUID] = ApiField(alias='owningUsers', default_factory=list)

class ApplicationRequest(ObjectModel):
    certificateIssuingTemplateAliasIdMap: Dict[str, UUID] = ApiField(
        alias='certificateIssuingTemplateAliasIdMap',
        default_factory=dict
    )
    description: str = ApiField(alias='description')
    externalIpRanges: List[str] = ApiField(alias='externalIpRanges', default_factory=list)
    fqdns: List[str] = ApiField(alias='fqdns', default_factory=list)
    fullyQualifiedDomainNames: List[str] = ApiField(alias='fullyQualifiedDomainNames', default_factory=list)
    internalFqdns: List[str] = ApiField(alias='internalFqdns', default_factory=list)
    internalIpRanges: List[str] = ApiField(alias='internalIpRanges', default_factory=list)
    internalPorts: List[str] = ApiField(alias='internalPorts', default_factory=list)
    ipRanges: List[str] = ApiField(alias='ipRanges', default_factory=list)
    name: str = ApiField(alias='name')
    ownerIdsAndTypes: List[OwnerIdAndType] = ApiField(alias='ownerIdsAndTypes', default_factory=list)
    ports: List[str] = ApiField(alias='ports', default_factory=list)
    startTargetedDiscovery: bool = ApiField(alias='startTargetedDiscovery')

class ApplicationResponse(ObjectModel):
    applications: List[ApplicationInformation] = ApiField(alias='applications', default_factory=list)
    ownershipCount: int = ApiField(alias='ownershipCount')
    totalCount: int = ApiField(alias='totalCount')

class ApplicationsAssignRequest(ObjectModel):
    action: Literal['ADD', 'DELETE', 'DELETE_ALL', 'REPLACE'] = ApiField(alias='action')
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)
    targetedApplicationIds: List[UUID] = ApiField(alias='targetedApplicationIds', default_factory=list)

class ApplicationsAssignResponse(ObjectModel):
    certificates: List[CertificateInformation] = ApiField(alias='certificates', default_factory=list)

class BaseFilter(ObjectModel):
    ...

class BaseOrdering(ObjectModel):
    orders: List[Order] = ApiField(alias='orders', default_factory=list)

class Bucket(ObjectModel):
    first: str = ApiField(alias='first')
    second: List[NestedFacet] = ApiField(alias='second', default_factory=list)

class CSRAttributesInformation(ObjectModel):
    commonName: str = ApiField(alias='commonName')
    country: str = ApiField(alias='country')
    keyTypeParameters: KeyTypeParameters = ApiField(alias='keyTypeParameters')
    locality: str = ApiField(alias='locality')
    organization: str = ApiField(alias='organization')
    organizationalUnits: List[str] = ApiField(alias='organizationalUnits', default_factory=list)
    state: str = ApiField(alias='state')
    subjectAlternativeNamesByType: SubjectAlternativeNamesByType = ApiField(alias='subjectAlternativeNamesByType')

class CertificateAggregatesRangeResponse(ObjectModel):
    certificateAggregatesMap: Dict[str, int] = ApiField(alias='certificateAggregatesMap', default_factory=dict)
    companyId: UUID = ApiField(alias='companyId')

class CertificateAggregatesResponse(ObjectModel):
    certificateAggregatesMap: Dict[str, CertificateCategoryInformation] = ApiField(
        alias='certificateAggregatesMap',
        default_factory=dict
    )
    companyId: UUID = ApiField(alias='companyId')

class CertificateAggregationsRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    facet: Facet = ApiField(alias='facet')
    includeRetired: bool = ApiField(alias='includeRetired')

class CertificateAuthorityProductInformation(ObjectModel):
    certificateType: Literal['DOMAIN_VALIDATED_SSL', 'OTHER'] = ApiField(alias='certificateType')
    hashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='hashAlgorithm')
    productName: str = ApiField(alias='productName')
    productTypes: List[Literal['CODESIGN', 'SSL']] = ApiField(alias='productTypes', default_factory=list)
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateCategoryInformation(ObjectModel):
    certificateCategoryAggregatesMap: Dict[str, int] = ApiField(
        alias='certificateCategoryAggregatesMap',
        default_factory=dict
    )
    certificatesCount: int = ApiField(alias='certificatesCount')

class CertificateDeletionRequest(ObjectModel):
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)

class CertificateImportInfo(ObjectModel):
    apiClientInformation: ApiClientInformation = ApiField(alias='apiClientInformation')
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    certificate: str = ApiField(alias='certificate')
    certificateUsageMetadata: List[CertificateUsageMetadata] = ApiField(
        alias='certificateUsageMetadata',
        default_factory=list
    )
    issuerCertificates: List[str] = ApiField(alias='issuerCertificates', default_factory=list)

class CertificateImportRequest(ObjectModel):
    certificates: List[CertificateImportInfo] = ApiField(alias='certificates', default_factory=list)
    overrideBlocklist: bool = ApiField(alias='overrideBlocklist')

class CertificateImportResponse(ObjectModel):
    certificateInformations: List[ImportedCertificateInformation] = ApiField(
        alias='certificateInformations',
        default_factory=list
    )
    statistics: Dict[str, int] = ApiField(alias='statistics', default_factory=dict)

class CertificateInformation(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    archivedDate: datetime = ApiField(alias='archivedDate')
    authorityKeyIdentifierHash: str = ApiField(alias='authorityKeyIdentifierHash')
    certificateAuthorityAccountId: UUID = ApiField(alias='certificateAuthorityAccountId')
    certificateName: str = ApiField(alias='certificateName')
    certificateRequestId: UUID = ApiField(alias='certificateRequestId')
    certificateStatus: Literal['ACTIVE', 'DELETED', 'RETIRED'] = ApiField(alias='certificateStatus')
    companyId: UUID = ApiField(alias='companyId')
    dekHash: str = ApiField(alias='dekHash')
    encryptionType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(
        alias='encryptionType'
    )
    extendedKeyUsage: List[str] = ApiField(alias='extendedKeyUsage', default_factory=list)
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    inhibitAnyPolicy: int = ApiField(alias='inhibitAnyPolicy')
    inhibitPolicyMapping: int = ApiField(alias='inhibitPolicyMapping')
    instances: List[CertificateInstanceInformation] = ApiField(alias='instances', default_factory=list)
    issuerAlternativeNameDns: List[str] = ApiField(alias='issuerAlternativeNameDns', default_factory=list)
    issuerAlternativeNameNonDns: List[str] = ApiField(alias='issuerAlternativeNameNonDns', default_factory=list)
    issuerC: str = ApiField(alias='issuerC')
    issuerCN: List[str] = ApiField(alias='issuerCN', default_factory=list)
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    issuerDN: str = ApiField(alias='issuerDN')
    issuerL: str = ApiField(alias='issuerL')
    issuerOU: List[str] = ApiField(alias='issuerOU', default_factory=list)
    issuerST: str = ApiField(alias='issuerST')
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyStrength: int = ApiField(alias='keyStrength')
    keyUsage: List[str] = ApiField(alias='keyUsage', default_factory=list)
    lastNotification: int = ApiField(alias='lastNotification')
    managedCertificateId: UUID = ApiField(alias='managedCertificateId')
    modificationDate: datetime = ApiField(alias='modificationDate')
    ocspNoCheck: bool = ApiField(alias='ocspNoCheck')
    ownership: CertificateOwnership = ApiField(alias='ownership')
    pathLength: int = ApiField(alias='pathLength')
    requireExplicitPolicy: int = ApiField(alias='requireExplicitPolicy')
    selfSigned: bool = ApiField(alias='selfSigned')
    serialNumber: str = ApiField(alias='serialNumber')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    signatureHashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='signatureHashAlgorithm')
    statusModificationDate: datetime = ApiField(alias='statusModificationDate')
    statusModificationUserId: UUID = ApiField(alias='statusModificationUserId')
    subjectAlternativeNameDns: List[str] = ApiField(alias='subjectAlternativeNameDns', default_factory=list)
    subjectAlternativeNameNonDns: List[str] = ApiField(alias='subjectAlternativeNameNonDns', default_factory=list)
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectC: str = ApiField(alias='subjectC')
    subjectCN: List[str] = ApiField(alias='subjectCN', default_factory=list)
    subjectDN: str = ApiField(alias='subjectDN')
    subjectKeyIdentifierHash: str = ApiField(alias='subjectKeyIdentifierHash')
    subjectL: str = ApiField(alias='subjectL')
    subjectO: str = ApiField(alias='subjectO')
    subjectOU: List[str] = ApiField(alias='subjectOU', default_factory=list)
    subjectST: str = ApiField(alias='subjectST')
    tags: List[str] = ApiField(alias='tags', default_factory=list)
    totalActiveInstanceCount: int = ApiField(alias='totalActiveInstanceCount')
    totalInstanceCount: int = ApiField(alias='totalInstanceCount')
    validityEnd: datetime = ApiField(alias='validityEnd')
    validityStart: datetime = ApiField(alias='validityStart')
    versionType: Literal['CURRENT', 'OLD'] = ApiField(alias='versionType')

class CertificateInstallationsUsageResponse(ObjectModel):
    count: int = ApiField(alias='count')

class CertificateInstanceInformation(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    certificateId: UUID = ApiField(alias='certificateId')
    certificateInstanceId: UUID = ApiField(alias='certificateInstanceId')
    certificateSource: Literal[
        'AWS_DISCOVERY', 'AZURE_DISCOVERY', 'DOMAIN_SCAN', 'EXTERNAL_CA_IMPORT', 'EXTERNAL_SCAN', 'FILE_IMPORT', 'GCP_DISCOVERY', 'KUBERNETES_DISCOVERY', 'MACHINE_DISCOVERY', 'ON_PREM_CA_IMPORT',
        'SMART_SCAN_EXTERNAL', 'SMART_SCAN_INTERNAL', 'SMART_VALIDATION_EXTERNAL', 'SMART_VALIDATION_INTERNAL', 'TRUSTNET_SCAN', 'UNKNOWN', 'USER_IMPORTED', 'USER_PROVIDED', 'USER_SCAN'] = ApiField(
        alias='certificateSource'
    )
    deploymentStatus: Literal['IN_USE', 'SUPERSEDED', 'UNKNOWN'] = ApiField(alias='deploymentStatus')
    hostname: str = ApiField(alias='hostname')
    instanceChainValidationStatus: List[
        Literal['CHAIN_BUILDING_FAILED', 'CHAIN_EXPIRE_BEFORE_EE', 'DISTRUSTED', 'INCOMPLETE_CHAIN',
        'OK', 'SELF_SIGNED', 'UNKNOWN_ERROR']] = ApiField(alias='instanceChainValidationStatus', default_factory=list)
    ipAddress: str = ApiField(alias='ipAddress')
    lastScanDate: datetime = ApiField(alias='lastScanDate')
    lastValidatedAttempt: datetime = ApiField(alias='lastValidatedAttempt')
    modificationDate: datetime = ApiField(alias='modificationDate')
    port: int = ApiField(alias='port')
    serviceIds: List[UUID] = ApiField(alias='serviceIds', default_factory=list)
    sslProtocols: List[str] = ApiField(alias='sslProtocols', default_factory=list)
    sslValidationErrorArguments: List[str] = ApiField(alias='sslValidationErrorArguments', default_factory=list)
    sslValidationStatus: Literal[
        'HOSTNAME_NOT_RESOLVABLE', 'INVALID_CERTIFICATE_FOUND', 'NO_CERTIFICATE_PRESENTED', 'OK',
        'OLD_VERSION_CERTIFICATE_FOUND', 'TARGET_UNREACHABLE', 'UNEXPECTED_CERTIFICATE_FOUND', 'UNKNOWN_ERROR'] = ApiField(
        alias='sslValidationStatus'
    )
    sslValidationStatusMessage: str = ApiField(alias='sslValidationStatusMessage')

class CertificateInstanceSearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class CertificateInstanceValidationRequest(ObjectModel):
    instanceIds: List[UUID] = ApiField(alias='instanceIds', default_factory=list)

class CertificateIssuingTemplateInformation(ObjectModel):
    certificateAuthority: Literal['ACME', 'BUILTIN', 'CONNECTOR', 'DIGICERT', 'ENTRUST', 'GLOBALSIGN',
    'GLOBALSIGNMSSL', 'MICROSOFT', 'MOCKCA', 'TPP', 'ZTPKI'] = ApiField(alias='certificateAuthority')
    certificateAuthorityAccountId: UUID = ApiField(alias='certificateAuthorityAccountId')
    certificateAuthorityProductOptionId: UUID = ApiField(alias='certificateAuthorityProductOptionId')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    csrUploadAllowed: bool = ApiField(alias='csrUploadAllowed')
    everyoneIsConsumer: bool = ApiField(alias='everyoneIsConsumer')
    id: UUID = ApiField(alias='id')
    keyGeneratedByVenafiAllowed: bool = ApiField(alias='keyGeneratedByVenafiAllowed')
    keyReuse: bool = ApiField(alias='keyReuse')
    keyTypes: List[KeyTypeInformation] = ApiField(alias='keyTypes', default_factory=list)
    modificationDate: datetime = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    priority: int = ApiField(alias='priority')
    product: CertificateAuthorityProductInformation = ApiField(alias='product')
    productEntitlement: Literal['ANY', 'CODESIGN', 'DEVOPS', 'MIRA', 'OUTAGE_DETECTION'] = ApiField(
        alias='productEntitlement'
    )
    productEntitlements: Literal['ANY', 'CODESIGN', 'DEVOPS', 'MIRA', 'OUTAGE_DETECTION'] = ApiField(
        alias='productEntitlements'
    )
    reason: str = ApiField(alias='reason')
    recommendedSettings: RecommendedSettingsInformation = ApiField(alias='recommendedSettings')
    resourceConsumerTeamIds: List[UUID] = ApiField(alias='resourceConsumerTeamIds', default_factory=list)
    resourceConsumerUserIds: List[UUID] = ApiField(alias='resourceConsumerUserIds', default_factory=list)
    sanDnsNameRegexes: List[str] = ApiField(alias='sanDnsNameRegexes', default_factory=list)
    sanIpAddressRegexes: List[str] = ApiField(alias='sanIpAddressRegexes', default_factory=list)
    sanRegexes: List[str] = ApiField(alias='sanRegexes', default_factory=list)
    sanRfc822NameRegexes: List[str] = ApiField(alias='sanRfc822NameRegexes', default_factory=list)
    sanUniformResourceIdentifierRegexes: List[str] = ApiField(
        alias='sanUniformResourceIdentifierRegexes',
        default_factory=list
    )
    status: Literal['AVAILABLE', 'UNAVAILABLE'] = ApiField(alias='status')
    subjectCNRegexes: List[str] = ApiField(alias='subjectCNRegexes', default_factory=list)
    subjectCValues: List[str] = ApiField(alias='subjectCValues', default_factory=list)
    subjectLRegexes: List[str] = ApiField(alias='subjectLRegexes', default_factory=list)
    subjectORegexes: List[str] = ApiField(alias='subjectORegexes', default_factory=list)
    subjectOURegexes: List[str] = ApiField(alias='subjectOURegexes', default_factory=list)
    subjectSTRegexes: List[str] = ApiField(alias='subjectSTRegexes', default_factory=list)
    systemGenerated: bool = ApiField(alias='systemGenerated')
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateKeystoreRequest(ObjectModel):
    certificateLabel: str = ApiField(alias='certificateLabel')
    encryptedKeystorePassphrase: str = ApiField(alias='encryptedKeystorePassphrase')
    encryptedPrivateKeyPassphrase: str = ApiField(alias='encryptedPrivateKeyPassphrase')
    exportFormat: Literal['JKS', 'PEM', 'PKCS12'] = ApiField(alias='exportFormat')

class CertificateOwnership(ObjectModel):
    owningContainers: List[ExtendedApplicationOwnership] = ApiField(alias='owningContainers', default_factory=list)

class CertificateRecoveryRequest(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)

class CertificateRequestDocumentInformation(ObjectModel):
    applicationId: UUID = ApiField(alias='applicationId')
    approvedIds: List[UUID] = ApiField(alias='approvedIds', default_factory=list)
    approverTeamIds: List[UUID] = ApiField(alias='approverTeamIds', default_factory=list)
    approverUserIds: List[UUID] = ApiField(alias='approverUserIds', default_factory=list)
    caOrderId: str = ApiField(alias='caOrderId')
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)
    certificateIssuingTemplateId: UUID = ApiField(alias='certificateIssuingTemplateId')
    certificateOwnerUserId: UUID = ApiField(alias='certificateOwnerUserId')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    customAttributes: CustomAttributes = ApiField(alias='customAttributes')
    errorInformation: ErrorInformation = ApiField(alias='errorInformation')
    finalApproverId: UUID = ApiField(alias='finalApproverId')
    id: UUID = ApiField(alias='id')
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyLength: int = ApiField(alias='keyLength')
    keyType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(alias='keyType')
    modificationDate: datetime = ApiField(alias='modificationDate')
    status: Literal['CANCELLED', 'DELETED', 'FAILED', 'ISSUED', 'NEW', 'PENDING', 'PENDING_APPROVAL',
    'PENDING_FINAL_APPROVAL', 'REJECTED', 'REJECTED_APPROVAL', 'REQUESTED', 'REVOKED'] = ApiField(alias='status')
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectCN: str = ApiField(alias='subjectCN')
    subjectDN: str = ApiField(alias='subjectDN')
    tags: List[str] = ApiField(alias='tags', default_factory=list)
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateRequestDocumentResponse(ObjectModel):
    certificateRequests: List[CertificateRequestDocumentInformation] = ApiField(
        alias='certificateRequests',
        default_factory=list
    )
    numFound: int = ApiField(alias='numFound')

class CertificateRequestInformation(ObjectModel):
    apiClientInformation: ApiClientInformation = ApiField(alias='apiClientInformation')
    applicationId: UUID = ApiField(alias='applicationId')
    approvedIds: List[UUID] = ApiField(alias='approvedIds', default_factory=list)
    approverTeamIds: List[UUID] = ApiField(alias='approverTeamIds', default_factory=list)
    approverUserIds: List[UUID] = ApiField(alias='approverUserIds', default_factory=list)
    caOrderId: str = ApiField(alias='caOrderId')
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)
    certificateIssuingTemplateId: UUID = ApiField(alias='certificateIssuingTemplateId')
    certificateOwnerUserId: UUID = ApiField(alias='certificateOwnerUserId')
    certificateSigningRequest: str = ApiField(alias='certificateSigningRequest')
    certificateUsageMetadata: List[CertificateUsageMetadata] = ApiField(
        alias='certificateUsageMetadata',
        default_factory=list
    )
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    customAttributesInformation: CustomAttributesInformation = ApiField(alias='customAttributesInformation')
    errorInformation: ErrorInformation = ApiField(alias='errorInformation')
    finalApproverId: UUID = ApiField(alias='finalApproverId')
    id: UUID = ApiField(alias='id')
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyLength: int = ApiField(alias='keyLength')
    keyType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(alias='keyType')
    modificationDate: datetime = ApiField(alias='modificationDate')
    status: Literal['CANCELLED', 'DELETED', 'FAILED', 'ISSUED', 'NEW', 'PENDING', 'PENDING_APPROVAL',
    'PENDING_FINAL_APPROVAL', 'REJECTED', 'REJECTED_APPROVAL', 'REQUESTED', 'REVOKED'] = ApiField(alias='status')
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectDN: str = ApiField(alias='subjectDN')
    tags: List[str] = ApiField(alias='tags', default_factory=list)
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateRequestRequest(ObjectModel):
    apiClientInformation: ApiClientInformation = ApiField(alias='apiClientInformation')
    applicationId: UUID = ApiField(alias='applicationId')
    certificateIssuingTemplateId: UUID = ApiField(alias='certificateIssuingTemplateId')
    certificateOwnerUserId: UUID = ApiField(alias='certificateOwnerUserId')
    certificateSigningRequest: str = ApiField(alias='certificateSigningRequest')
    certificateUsageMetadata: List[CertificateUsageMetadata] = ApiField(
        alias='certificateUsageMetadata',
        default_factory=list
    )
    csrAttributes: CSRAttributesInformation = ApiField(alias='csrAttributes')
    customAttributes: CustomAttributesInformation = ApiField(alias='customAttributes')
    existingCertificateId: UUID = ApiField(alias='existingCertificateId')
    isVaaSGenerated: bool = ApiField(alias='isVaaSGenerated')
    reuseCSR: bool = ApiField(alias='reuseCSR')
    tags: List[str] = ApiField(alias='tags', default_factory=list)
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateRequestResponse(ObjectModel):
    certificateRequests: List[CertificateRequestInformation] = ApiField(
        alias='certificateRequests',
        default_factory=list
    )

class CertificateRequestResubmissionRequest(ObjectModel):
    certificateIssuingTemplateId: UUID = ApiField(alias='certificateIssuingTemplateId')
    certificateOwnerUserId: UUID = ApiField(alias='certificateOwnerUserId')
    certificateUsageMetadata: List[CertificateUsageMetadata] = ApiField(
        alias='certificateUsageMetadata',
        default_factory=list
    )
    validityPeriod: str = ApiField(alias='validityPeriod')

class CertificateRequestsSearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class CertificateResponse(ObjectModel):
    certificates: List[CertificateInformation] = ApiField(alias='certificates', default_factory=list)
    count: int = ApiField(alias='count')

class CertificateRetirementRequest(ObjectModel):
    addToBlocklist: bool = ApiField(alias='addToBlocklist')
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)

class CertificateSearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class CertificateUsageMetadata(ObjectModel):
    appName: str = ApiField(alias='appName')
    automationMetadata: str = ApiField(alias='automationMetadata')
    nodeName: str = ApiField(alias='nodeName')

class CertificateValidationRequest(ObjectModel):
    certificateIds: List[UUID] = ApiField(alias='certificateIds', default_factory=list)

class CertificateValidationResponse(ObjectModel):
    fingerprints: List[str] = ApiField(alias='fingerprints', default_factory=list)

class CertificationRequestInformation(ObjectModel):
    hashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='hashAlgorithm')
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyLength: int = ApiField(alias='keyLength')
    keyType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(alias='keyType')
    publicKeyHash: str = ApiField(alias='publicKeyHash')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectDN: str = ApiField(alias='subjectDN')

class Condition(ObjectModel):
    field: str = ApiField(alias='field')
    operator: Literal['EQ', 'FIND', 'GT', 'GTE', 'IN', 'LT', 'LTE', 'MATCH'] = ApiField(alias='operator')
    value: str = ApiField(alias='value')
    values: List[str] = ApiField(alias='values', default_factory=list)

class CustomAttributes(ObjectModel):
    dnsNames: List[str] = ApiField(alias='dnsNames', default_factory=list)
    overwriteSans: bool = ApiField(alias='overwriteSans')

class CustomAttributesInformation(ObjectModel):
    dnsNames: List[str] = ApiField(alias='dnsNames', default_factory=list)
    overwriteSans: bool = ApiField(alias='overwriteSans')

class EntityTag(ObjectModel):
    value: str = ApiField(alias='value')
    weak: bool = ApiField(alias='weak')

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class Expression(ObjectModel):
    ...

class ExtendedApplicationOwnership(ObjectModel):
    id: UUID = ApiField(alias='id')
    owningTeams: List[UUID] = ApiField(alias='owningTeams', default_factory=list)
    owningUsers: List[UUID] = ApiField(alias='owningUsers', default_factory=list)
    type: str = ApiField(alias='type')

class ExtendedCertificateInformation(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    archivedDate: datetime = ApiField(alias='archivedDate')
    authorityKeyIdentifierHash: str = ApiField(alias='authorityKeyIdentifierHash')
    certificateAuthorityAccountId: UUID = ApiField(alias='certificateAuthorityAccountId')
    certificateName: str = ApiField(alias='certificateName')
    certificateRequestId: UUID = ApiField(alias='certificateRequestId')
    certificateStatus: Literal['ACTIVE', 'DELETED', 'RETIRED'] = ApiField(alias='certificateStatus')
    companyId: UUID = ApiField(alias='companyId')
    dekHash: str = ApiField(alias='dekHash')
    encryptionType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(
        alias='encryptionType'
    )
    extendedKeyUsage: List[str] = ApiField(alias='extendedKeyUsage', default_factory=list)
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    inhibitAnyPolicy: int = ApiField(alias='inhibitAnyPolicy')
    inhibitPolicyMapping: int = ApiField(alias='inhibitPolicyMapping')
    instances: List[CertificateInstanceInformation] = ApiField(alias='instances', default_factory=list)
    issuerAlternativeNameDns: List[str] = ApiField(alias='issuerAlternativeNameDns', default_factory=list)
    issuerAlternativeNameNonDns: List[str] = ApiField(alias='issuerAlternativeNameNonDns', default_factory=list)
    issuerC: str = ApiField(alias='issuerC')
    issuerCN: List[str] = ApiField(alias='issuerCN', default_factory=list)
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    issuerCertificates: List[CertificateInformation] = ApiField(alias='issuerCertificates', default_factory=list)
    issuerDN: str = ApiField(alias='issuerDN')
    issuerL: str = ApiField(alias='issuerL')
    issuerOU: List[str] = ApiField(alias='issuerOU', default_factory=list)
    issuerST: str = ApiField(alias='issuerST')
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyStrength: int = ApiField(alias='keyStrength')
    keyUsage: List[str] = ApiField(alias='keyUsage', default_factory=list)
    lastNotification: int = ApiField(alias='lastNotification')
    managedCertificateId: UUID = ApiField(alias='managedCertificateId')
    modificationDate: datetime = ApiField(alias='modificationDate')
    ocspNoCheck: bool = ApiField(alias='ocspNoCheck')
    ownership: CertificateOwnership = ApiField(alias='ownership')
    pathLength: int = ApiField(alias='pathLength')
    requireExplicitPolicy: int = ApiField(alias='requireExplicitPolicy')
    selfSigned: bool = ApiField(alias='selfSigned')
    serialNumber: str = ApiField(alias='serialNumber')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    signatureHashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='signatureHashAlgorithm')
    statusModificationDate: datetime = ApiField(alias='statusModificationDate')
    statusModificationUserId: UUID = ApiField(alias='statusModificationUserId')
    subjectAlternativeNameDns: List[str] = ApiField(alias='subjectAlternativeNameDns', default_factory=list)
    subjectAlternativeNameNonDns: List[str] = ApiField(alias='subjectAlternativeNameNonDns', default_factory=list)
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectC: str = ApiField(alias='subjectC')
    subjectCN: List[str] = ApiField(alias='subjectCN', default_factory=list)
    subjectDN: str = ApiField(alias='subjectDN')
    subjectKeyIdentifierHash: str = ApiField(alias='subjectKeyIdentifierHash')
    subjectL: str = ApiField(alias='subjectL')
    subjectO: str = ApiField(alias='subjectO')
    subjectOU: List[str] = ApiField(alias='subjectOU', default_factory=list)
    subjectST: str = ApiField(alias='subjectST')
    tags: List[str] = ApiField(alias='tags', default_factory=list)
    totalActiveInstanceCount: int = ApiField(alias='totalActiveInstanceCount')
    totalInstanceCount: int = ApiField(alias='totalInstanceCount')
    validityEnd: datetime = ApiField(alias='validityEnd')
    validityStart: datetime = ApiField(alias='validityStart')
    versionType: Literal['CURRENT', 'OLD'] = ApiField(alias='versionType')

class ExtendedCertificateInstanceInformation(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    certificate: CertificateInformation = ApiField(alias='certificate')
    certificateId: UUID = ApiField(alias='certificateId')
    certificateInstanceId: UUID = ApiField(alias='certificateInstanceId')
    certificateSource: Literal[
        'AWS_DISCOVERY', 'AZURE_DISCOVERY', 'DOMAIN_SCAN', 'EXTERNAL_CA_IMPORT', 'EXTERNAL_SCAN', 'FILE_IMPORT', 'GCP_DISCOVERY', 'KUBERNETES_DISCOVERY', 'MACHINE_DISCOVERY', 'ON_PREM_CA_IMPORT',
        'SMART_SCAN_EXTERNAL', 'SMART_SCAN_INTERNAL', 'SMART_VALIDATION_EXTERNAL', 'SMART_VALIDATION_INTERNAL', 'TRUSTNET_SCAN', 'UNKNOWN', 'USER_IMPORTED', 'USER_PROVIDED', 'USER_SCAN'] = ApiField(
        alias='certificateSource'
    )
    deploymentStatus: Literal['IN_USE', 'SUPERSEDED', 'UNKNOWN'] = ApiField(alias='deploymentStatus')
    hostname: str = ApiField(alias='hostname')
    instanceChainValidationStatus: List[
        Literal['CHAIN_BUILDING_FAILED', 'CHAIN_EXPIRE_BEFORE_EE', 'DISTRUSTED', 'INCOMPLETE_CHAIN',
        'OK', 'SELF_SIGNED', 'UNKNOWN_ERROR']] = ApiField(alias='instanceChainValidationStatus', default_factory=list)
    ipAddress: str = ApiField(alias='ipAddress')
    lastScanDate: datetime = ApiField(alias='lastScanDate')
    lastValidatedAttempt: datetime = ApiField(alias='lastValidatedAttempt')
    modificationDate: datetime = ApiField(alias='modificationDate')
    port: int = ApiField(alias='port')
    serviceIds: List[UUID] = ApiField(alias='serviceIds', default_factory=list)
    sslProtocols: List[str] = ApiField(alias='sslProtocols', default_factory=list)
    sslValidationErrorArguments: List[str] = ApiField(alias='sslValidationErrorArguments', default_factory=list)
    sslValidationStatus: Literal[
        'HOSTNAME_NOT_RESOLVABLE', 'INVALID_CERTIFICATE_FOUND', 'NO_CERTIFICATE_PRESENTED', 'OK',
        'OLD_VERSION_CERTIFICATE_FOUND', 'TARGET_UNREACHABLE', 'UNEXPECTED_CERTIFICATE_FOUND', 'UNKNOWN_ERROR'] = ApiField(
        alias='sslValidationStatus'
    )
    sslValidationStatusMessage: str = ApiField(alias='sslValidationStatusMessage')

class ExtendedCertificateInstanceResponse(ObjectModel):
    count: int = ApiField(alias='count')
    instances: List[ExtendedCertificateInstanceInformation] = ApiField(alias='instances', default_factory=list)

class Facet(ObjectModel):
    domain: Dict[str, AnyValue] = ApiField(alias='domain', default_factory=dict)
    facets: List[Facet] = ApiField(alias='facets', default_factory=list)
    name: str = ApiField(alias='name')

class FacetResponse(ObjectModel):
    aggregates: Dict[str, str] = ApiField(alias='aggregates', default_factory=dict)
    buckets: List[Bucket] = ApiField(alias='buckets', default_factory=list)

class Filter(ObjectModel):
    expression: BaseFilter = ApiField(alias='expression')
    ordering: BaseOrdering = ApiField(alias='ordering')
    paging: Page = ApiField(alias='paging')

class FilterOperand(ObjectModel):
    operand: Condition = ApiField(alias='operand')
    operator: Literal['NOT'] = ApiField(alias='operator')

class FilterOperands(ObjectModel):
    operands: List[BaseFilter] = ApiField(alias='operands', default_factory=list)
    operator: Literal['AND', 'OR'] = ApiField(alias='operator')

class FlattenedCertificateSavedSearch(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class GeneralNamesData(ObjectModel):
    dNSName: List[str] = ApiField(alias='dNSName', default_factory=list)
    directoryName: List[str] = ApiField(alias='directoryName', default_factory=list)
    ediPartyName: List[str] = ApiField(alias='ediPartyName', default_factory=list)
    iPAddress: List[str] = ApiField(alias='iPAddress', default_factory=list)
    otherName: List[str] = ApiField(alias='otherName', default_factory=list)
    registeredID: List[str] = ApiField(alias='registeredID', default_factory=list)
    rfc822Name: List[str] = ApiField(alias='rfc822Name', default_factory=list)
    uniformResourceIdentifier: List[str] = ApiField(alias='uniformResourceIdentifier', default_factory=list)
    x400Address: List[str] = ApiField(alias='x400Address', default_factory=list)

class ImportedCertificateInformation(ObjectModel):
    apiClientInformation: ApiClientInformation = ApiField(alias='apiClientInformation')
    base64Certificate: str = ApiField(alias='base64Certificate')
    certificateSource: Literal[
        'AWS_DISCOVERY', 'AZURE_DISCOVERY', 'DOMAIN_SCAN', 'EXTERNAL_CA_IMPORT', 'EXTERNAL_SCAN', 'FILE_IMPORT', 'GCP_DISCOVERY', 'KUBERNETES_DISCOVERY', 'MACHINE_DISCOVERY', 'ON_PREM_CA_IMPORT',
        'SMART_SCAN_EXTERNAL', 'SMART_SCAN_INTERNAL', 'SMART_VALIDATION_EXTERNAL', 'SMART_VALIDATION_INTERNAL', 'TRUSTNET_SCAN', 'UNKNOWN', 'USER_IMPORTED', 'USER_PROVIDED', 'USER_SCAN'] = ApiField(
        alias='certificateSource'
    )
    certificateUsageMetadata: List[CertificateUsageMetadata] = ApiField(
        alias='certificateUsageMetadata',
        default_factory=list
    )
    companyId: UUID = ApiField(alias='companyId')
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    managedCertificateId: UUID = ApiField(alias='managedCertificateId')

class InventoryMonitoringConfig(ObjectModel):
    inventoryMonitoringConfigurationType: str = ApiField(alias='inventoryMonitoringConfigurationType')

class InventoryMonitoringConfigRequest(ObjectModel):
    inventoryMonitoringConfiguration: Union[CertificateExpirationInventoryMonitoringConfig] = ApiField(
        alias='inventoryMonitoringConfiguration'
    )

class InventoryMonitoringConfiguration(ObjectModel):
    inventoryMonitoringConfigurationType: str = ApiField(alias='inventoryMonitoringConfigurationType')

class InventoryMonitoringConfigurationRequest(ObjectModel):
    inventoryMonitoringConfiguration: InventoryMonitoringConfiguration = ApiField(
        alias='inventoryMonitoringConfiguration'
    )

class InventoryMonitoringConfigurationResponse(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    configuration: InventoryMonitoringConfiguration = ApiField(alias='configuration')
    id: UUID = ApiField(alias='id')

class InventoryMonitoringSchedulerInformation(ObjectModel):
    name: str = ApiField(alias='name')
    status: Literal['DONE', 'FAILED', 'PENDING', 'RUNNING'] = ApiField(alias='status')

class InvitationInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    id: UUID = ApiField(alias='id')
    productRoles: Dict[str, List[Literal['DEVOPS_LEAD', 'DEVOPS_USER', 'GUEST', 'OUTAGEDETECTION_ADMIN', 'PKI_ADMIN',
    'PLATFORM_ADMIN', 'RESOURCE_OWNER', 'SECURITY_ADMIN']]] = ApiField(alias='productRoles', default_factory=dict)
    properties: Dict[str, str] = ApiField(alias='properties', default_factory=dict)
    userId: UUID = ApiField(alias='userId')

class InvitationRequest(ObjectModel):
    role: Literal['DEVOPS_LEAD', 'DEVOPS_USER', 'GUEST', 'OUTAGEDETECTION_ADMIN', 'PKI_ADMIN',
    'PLATFORM_ADMIN', 'RESOURCE_OWNER', 'SECURITY_ADMIN'] = ApiField(alias='role')

class InvitationResponse(ObjectModel):
    invitations: List[InvitationInformation] = ApiField(alias='invitations', default_factory=list)

class KeyTypeInformation(ObjectModel):
    keyType: str = ApiField(alias='keyType')

class KeyTypeParameters(ObjectModel):
    keyCurve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='keyCurve')
    keyLength: int = ApiField(alias='keyLength')
    keyType: Literal['EC', 'RSA'] = ApiField(alias='keyType')

class Link(ObjectModel):
    params: Dict[str, str] = ApiField(alias='params', default_factory=dict)
    rel: str = ApiField(alias='rel')
    rels: List[str] = ApiField(alias='rels', default_factory=list)
    title: str = ApiField(alias='title')
    type: str = ApiField(alias='type')
    uri: str = ApiField(alias='uri')
    uriBuilder: UriBuilder = ApiField(alias='uriBuilder')

class MediaType(ObjectModel):
    parameters: Dict[str, str] = ApiField(alias='parameters', default_factory=dict)
    subtype: str = ApiField(alias='subtype')
    type: str = ApiField(alias='type')
    wildcardSubtype: bool = ApiField(alias='wildcardSubtype')
    wildcardType: bool = ApiField(alias='wildcardType')

class MetadataInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    productEntitlement: Literal['ANY', 'CODESIGN', 'DEVOPS', 'MIRA', 'OUTAGE_DETECTION'] = ApiField(
        alias='productEntitlement'
    )

class MultivaluedMapStringObject(ObjectModel):
    empty: bool = ApiField(alias='empty')

class MultivaluedMapStringString(ObjectModel):
    empty: bool = ApiField(alias='empty')

class NestedBucket(ObjectModel):
    first: str = ApiField(alias='first')
    second: List[AnyValue] = ApiField(alias='second', default_factory=list)

class NestedFacet(ObjectModel):
    aggregates: Dict[str, str] = ApiField(alias='aggregates', default_factory=dict)
    buckets: List[NestedBucket] = ApiField(alias='buckets', default_factory=list)

class NewCookie(ObjectModel):
    comment: str = ApiField(alias='comment')
    domain: str = ApiField(alias='domain')
    expiry: datetime = ApiField(alias='expiry')
    httpOnly: bool = ApiField(alias='httpOnly')
    maxAge: int = ApiField(alias='maxAge')
    name: str = ApiField(alias='name')
    path: str = ApiField(alias='path')
    sameSite: Literal['LAX', 'NONE', 'STRICT'] = ApiField(alias='sameSite')
    secure: bool = ApiField(alias='secure')
    value: str = ApiField(alias='value')
    version: int = ApiField(alias='version')

class Order(ObjectModel):
    direction: Literal['ASC', 'DESC'] = ApiField(alias='direction')
    field: str = ApiField(alias='field')

class OrderObject(ObjectModel):
    direction: Literal['ASC', 'DESC'] = ApiField(alias='direction')
    field: str = ApiField(alias='field')

class Ordering(ObjectModel):
    orders: List[OrderObject] = ApiField(alias='orders', default_factory=list)

class OwnerIdAndType(ObjectModel):
    ownerId: UUID = ApiField(alias='ownerId')
    ownerType: Literal['SERVICE_ACCOUNT', 'TEAM', 'USER'] = ApiField(alias='ownerType')

class Page(ObjectModel):
    pageNumber: int = ApiField(alias='pageNumber')
    pageSize: int = ApiField(alias='pageSize')

class Paging(ObjectModel):
    pageNumber: int = ApiField(alias='pageNumber')
    pageSize: int = ApiField(alias='pageSize')

class ProviderConfigInformation(ObjectModel):
    api_key: str = ApiField(alias='api_key')
    endpoint: str = ApiField(alias='endpoint')

class ProviderInformation(ObjectModel):
    config: ProviderConfigInformation = ApiField(alias='config')
    inputs: List[ProviderInputInformation] = ApiField(alias='inputs', default_factory=list)
    type: str = ApiField(alias='type')

class ProviderInputInformation(ObjectModel):
    hosts: List[str] = ApiField(alias='hosts', default_factory=list)
    ports: List[str] = ApiField(alias='ports', default_factory=list)
    subnet: str = ApiField(alias='subnet')
    type: str = ApiField(alias='type')

class RSAKeyTypeInformation(KeyTypeInformation):
    keyLengths: List[int] = ApiField(alias='keyLengths', default_factory=list)

class RecommendedSettingsInformation(ObjectModel):
    key: RecommendedSettingsKeyTypeInformation = ApiField(alias='key')
    sanRegexes: List[str] = ApiField(alias='sanRegexes', default_factory=list)
    subjectCNRegexes: List[str] = ApiField(alias='subjectCNRegexes', default_factory=list)
    subjectCValue: str = ApiField(alias='subjectCValue')
    subjectLValue: str = ApiField(alias='subjectLValue')
    subjectOUValue: str = ApiField(alias='subjectOUValue')
    subjectOValue: str = ApiField(alias='subjectOValue')
    subjectSTValue: str = ApiField(alias='subjectSTValue')

class RecommendedSettingsKeyTypeInformation(ObjectModel):
    curve: Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN'] = ApiField(alias='curve')
    length: int = ApiField(alias='length')
    type: Literal['EC', 'RSA'] = ApiField(alias='type')

class Response(ObjectModel):
    allowedMethods: List[str] = ApiField(alias='allowedMethods', default_factory=list)
    closed: bool = ApiField(alias='closed')
    cookies: Dict[str, NewCookie] = ApiField(alias='cookies', default_factory=dict)
    date: datetime = ApiField(alias='date')
    entity: AnyValue = ApiField(alias='entity')
    entityTag: EntityTag = ApiField(alias='entityTag')
    headers: Dict[str, Any] = ApiField(alias='headers', default_factory=dict)
    language: Dict[str, Any] = ApiField(alias='language', default_factory=dict)
    lastModified: datetime = ApiField(alias='lastModified')
    length: int = ApiField(alias='length')
    links: List[Link] = ApiField(alias='links', default_factory=list)
    location: str = ApiField(alias='location')
    mediaType: MediaType = ApiField(alias='mediaType')
    metadata: Dict[str, Any] = ApiField(alias='metadata', default_factory=dict)
    status: int = ApiField(alias='status')
    statusInfo: StatusType = ApiField(alias='statusInfo')
    stringHeaders: Dict[str, Any] = ApiField(alias='stringHeaders', default_factory=dict)

class SavedSearchInfo(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    id: UUID = ApiField(alias='id')
    isDefault: bool = ApiField(alias='isDefault')
    name: str = ApiField(alias='name')
    searchDetails: FlattenedCertificateSavedSearch = ApiField(alias='searchDetails')
    type: Literal['FULL_TEXT', 'PREDEFINED_FILTER'] = ApiField(alias='type')
    userId: UUID = ApiField(alias='userId')

class SavedSearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    isDefault: bool = ApiField(alias='isDefault')
    name: str = ApiField(alias='name')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')
    type: Literal['FULL_TEXT', 'PREDEFINED_FILTER'] = ApiField(alias='type')

class SavedSearchResponse(ObjectModel):
    savedSearchInfo: List[SavedSearchInfo] = ApiField(alias='savedSearchInfo', default_factory=list)

class ScanafiConfigResponseV1(ObjectModel):
    id: str = ApiField(alias='id')
    metadata: MetadataInformation = ApiField(alias='metadata')
    provider: ProviderInformation = ApiField(alias='provider')

class ServiceAccountInformation(ObjectModel):
    applications: List[UUID] = ApiField(alias='applications', default_factory=list)
    applicationsAdded: List[UUID] = ApiField(alias='applicationsAdded', default_factory=list)
    applicationsRemoved: List[UUID] = ApiField(alias='applicationsRemoved', default_factory=list)
    authType: str = ApiField(alias='authType')
    authenticationType: str = ApiField(alias='authenticationType')
    companyId: UUID = ApiField(alias='companyId')
    enabled: bool = ApiField(alias='enabled')
    id: UUID = ApiField(alias='id')
    name: str = ApiField(alias='name')
    owner: UUID = ApiField(alias='owner')
    ownership: Dict[str, List[UUID]] = ApiField(alias='ownership', default_factory=dict)
    roles: List[str] = ApiField(alias='roles', default_factory=list)
    scopes: List[str] = ApiField(alias='scopes', default_factory=list)
    tenantId: UUID = ApiField(alias='tenantId')

class StatusType(ObjectModel):
    family: Literal['CLIENT_ERROR', 'INFORMATIONAL', 'OTHER', 'REDIRECTION', 'SERVER_ERROR', 'SUCCESSFUL'] = ApiField(
        alias='family'
    )
    reasonPhrase: str = ApiField(alias='reasonPhrase')
    statusCode: int = ApiField(alias='statusCode')

class SubjectAlternativeNamesByType(ObjectModel):
    dnsNames: List[str] = ApiField(alias='dnsNames', default_factory=list)
    ipAddresses: List[str] = ApiField(alias='ipAddresses', default_factory=list)
    rfc822Names: List[str] = ApiField(alias='rfc822Names', default_factory=list)
    uniformResourceIdentifiers: List[str] = ApiField(alias='uniformResourceIdentifiers', default_factory=list)

class TeamInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    id: UUID = ApiField(alias='id')
    members: List[UUID] = ApiField(alias='members', default_factory=list)
    name: str = ApiField(alias='name')
    owners: List[UUID] = ApiField(alias='owners', default_factory=list)
    ownership: Dict[str, List[UUID]] = ApiField(alias='ownership', default_factory=dict)
    productRoles: Dict[str, List[Literal['DEVOPS_LEAD', 'DEVOPS_USER', 'GUEST', 'OUTAGEDETECTION_ADMIN', 'PKI_ADMIN',
    'PLATFORM_ADMIN', 'RESOURCE_OWNER', 'SECURITY_ADMIN']]] = ApiField(alias='productRoles', default_factory=dict)
    systemRoles: List[Literal['CONDOR_METRICS', 'SYSTEM_ADMIN']] = ApiField(alias='systemRoles', default_factory=list)

class UnassignedCertificateAggregatesResponse(ObjectModel):
    certificateAggregatesMap: Dict[str, int] = ApiField(alias='certificateAggregatesMap', default_factory=dict)
    companyId: UUID = ApiField(alias='companyId')

class UriBuilder(ObjectModel):
    ...

class UserInformation(ObjectModel):
    admin: bool = ApiField(alias='admin')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    firstname: str = ApiField(alias='firstname')
    id: UUID = ApiField(alias='id')
    lastname: str = ApiField(alias='lastname')
    memberedTeams: List[UUID] = ApiField(alias='memberedTeams', default_factory=list)
    ownedTeams: List[UUID] = ApiField(alias='ownedTeams', default_factory=list)
    productRoles: Dict[str, List[Literal['DEVOPS_LEAD', 'DEVOPS_USER', 'GUEST', 'OUTAGEDETECTION_ADMIN', 'PKI_ADMIN',
    'PLATFORM_ADMIN', 'RESOURCE_OWNER', 'SECURITY_ADMIN']]] = ApiField(alias='productRoles', default_factory=dict)
    resourceOwner: bool = ApiField(alias='resourceOwner')
    serviceAccount: bool = ApiField(alias='serviceAccount')
    systemRoles: List[Literal['CONDOR_METRICS', 'SYSTEM_ADMIN']] = ApiField(alias='systemRoles', default_factory=list)
    teamsIds: List[UUID] = ApiField(alias='teamsIds', default_factory=list)
    userAccountType: str = ApiField(alias='userAccountType')
    userStatus: str = ApiField(alias='userStatus')
    userType: str = ApiField(alias='userType')
    username: str = ApiField(alias='username')

class CertificateExpirationInventoryMonitoringConfig(InventoryMonitoringConfig):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    enabled: bool = ApiField(alias='enabled')
    includeUnassignedCertificates: bool = ApiField(alias='includeUnassignedCertificates')
    thresholds: List[int] = ApiField(alias='thresholds', default_factory=list)

class CertificateExpirationInventoryMonitoringConfiguration(InventoryMonitoringConfiguration):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    enabled: bool = ApiField(alias='enabled')
    includeUnassignedCertificates: bool = ApiField(alias='includeUnassignedCertificates')
    thresholds: List[int] = ApiField(alias='thresholds', default_factory=list)

class ECKeyTypeInformation(KeyTypeInformation):
    keyCurves: List[Literal['ED25519', 'P256', 'P384', 'P521', 'UNKNOWN']] = ApiField(
        alias='keyCurves',
        default_factory=list
    )

ApiClientInformation.update_forward_refs()
ApplicationInformation.update_forward_refs()
ApplicationOwnership.update_forward_refs()
ApplicationRequest.update_forward_refs()
ApplicationResponse.update_forward_refs()
ApplicationsAssignRequest.update_forward_refs()
ApplicationsAssignResponse.update_forward_refs()
BaseFilter.update_forward_refs()
BaseOrdering.update_forward_refs()
Bucket.update_forward_refs()
CSRAttributesInformation.update_forward_refs()
CertificateAggregatesRangeResponse.update_forward_refs()
CertificateAggregatesResponse.update_forward_refs()
CertificateAggregationsRequest.update_forward_refs()
CertificateAuthorityProductInformation.update_forward_refs()
CertificateCategoryInformation.update_forward_refs()
CertificateDeletionRequest.update_forward_refs()
CertificateExpirationInventoryMonitoringConfig.update_forward_refs()
CertificateExpirationInventoryMonitoringConfiguration.update_forward_refs()
CertificateImportInfo.update_forward_refs()
CertificateImportRequest.update_forward_refs()
CertificateImportResponse.update_forward_refs()
CertificateInformation.update_forward_refs()
CertificateInstallationsUsageResponse.update_forward_refs()
CertificateInstanceInformation.update_forward_refs()
CertificateInstanceSearchRequest.update_forward_refs()
CertificateInstanceValidationRequest.update_forward_refs()
CertificateIssuingTemplateInformation.update_forward_refs()
CertificateKeystoreRequest.update_forward_refs()
CertificateOwnership.update_forward_refs()
CertificateRecoveryRequest.update_forward_refs()
CertificateRequestDocumentInformation.update_forward_refs()
CertificateRequestDocumentResponse.update_forward_refs()
CertificateRequestInformation.update_forward_refs()
CertificateRequestRequest.update_forward_refs()
CertificateRequestResponse.update_forward_refs()
CertificateRequestResubmissionRequest.update_forward_refs()
CertificateRequestsSearchRequest.update_forward_refs()
CertificateResponse.update_forward_refs()
CertificateRetirementRequest.update_forward_refs()
CertificateSearchRequest.update_forward_refs()
CertificateUsageMetadata.update_forward_refs()
CertificateValidationRequest.update_forward_refs()
CertificateValidationResponse.update_forward_refs()
CertificationRequestInformation.update_forward_refs()
Condition.update_forward_refs()
CustomAttributes.update_forward_refs()
CustomAttributesInformation.update_forward_refs()
ECKeyTypeInformation.update_forward_refs()
EntityTag.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
Expression.update_forward_refs()
ExtendedApplicationOwnership.update_forward_refs()
ExtendedCertificateInformation.update_forward_refs()
ExtendedCertificateInstanceInformation.update_forward_refs()
ExtendedCertificateInstanceResponse.update_forward_refs()
Facet.update_forward_refs()
FacetResponse.update_forward_refs()
Filter.update_forward_refs()
FilterOperand.update_forward_refs()
FilterOperands.update_forward_refs()
FlattenedCertificateSavedSearch.update_forward_refs()
GeneralNamesData.update_forward_refs()
ImportedCertificateInformation.update_forward_refs()
InventoryMonitoringConfig.update_forward_refs()
InventoryMonitoringConfigRequest.update_forward_refs()
InventoryMonitoringConfiguration.update_forward_refs()
InventoryMonitoringConfigurationRequest.update_forward_refs()
InventoryMonitoringConfigurationResponse.update_forward_refs()
InventoryMonitoringSchedulerInformation.update_forward_refs()
InvitationInformation.update_forward_refs()
InvitationRequest.update_forward_refs()
InvitationResponse.update_forward_refs()
KeyTypeInformation.update_forward_refs()
KeyTypeParameters.update_forward_refs()
Link.update_forward_refs()
MediaType.update_forward_refs()
MetadataInformation.update_forward_refs()
MultivaluedMapStringObject.update_forward_refs()
MultivaluedMapStringString.update_forward_refs()
NestedBucket.update_forward_refs()
NestedFacet.update_forward_refs()
NewCookie.update_forward_refs()
Order.update_forward_refs()
OrderObject.update_forward_refs()
Ordering.update_forward_refs()
OwnerIdAndType.update_forward_refs()
Page.update_forward_refs()
Paging.update_forward_refs()
ProviderConfigInformation.update_forward_refs()
ProviderInformation.update_forward_refs()
ProviderInputInformation.update_forward_refs()
RSAKeyTypeInformation.update_forward_refs()
RecommendedSettingsInformation.update_forward_refs()
RecommendedSettingsKeyTypeInformation.update_forward_refs()
Response.update_forward_refs()
SavedSearchInfo.update_forward_refs()
SavedSearchRequest.update_forward_refs()
SavedSearchResponse.update_forward_refs()
ScanafiConfigResponseV1.update_forward_refs()
ServiceAccountInformation.update_forward_refs()
StatusType.update_forward_refs()
SubjectAlternativeNamesByType.update_forward_refs()
TeamInformation.update_forward_refs()
UnassignedCertificateAggregatesResponse.update_forward_refs()
UriBuilder.update_forward_refs()
UserInformation.update_forward_refs()



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.identity import Identity
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

class Result(ObjectModel):
    code: int = ApiField()

    @property
    def recycle_bin_result(self) -> str:
        return ResultCodes.RecycleBin.get(self.code, 'Unknown')

class Deletion(ObjectModel):
    allotted_time: int = ApiField(alias='AllottedTime')
    query_size: int = ApiField(alias='QuerySize')
    sql_timeout: int = ApiField(alias='SqlTimeout')
    tasks: list[str] = ApiField(alias='Tasks', default_factory=list)

class Item(ObjectModel):
    child_count: int = ApiField(alias='ChildCount')
    dn: str = ApiField(alias='DN')
    deleted_by: Identity = ApiField(alias='DeletedBy')
    deleted_on: datetime = ApiField(alias='DeletedOn')
    guid: str = ApiField(alias='Guid')
    name: str = ApiField(alias='Name')
    purge_on: datetime = ApiField(alias='PurgeOn')
    restoration_dn: str = ApiField(alias='RestorationDN')
    restore_state: int = ApiField(alias='RestoreState')
    type: str = ApiField(alias='Type')
    type_detail: str = ApiField(alias='TypeDetail')

class Purge(ObjectModel):
    after: int = ApiField(alias='After')
    allotted_time: int = ApiField(alias='AllottedTime')
    engine_dn: str = ApiField(alias='EngineDN')

class Detail(ObjectModel):
    type: int = ApiField(alias='Type')
    vault_type: int = ApiField(alias='VaultType')
    config_class: str = ApiField(alias='ConfigClass')
    dn: str = ApiField(alias='Dn')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.client_group_base import ClientGroupBaseAttributes

class ClientGroupAttributes(ClientGroupBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Client Group"
    agent_type = Attribute('Agent Type')



from __future__ import annotations

import time
from typing import (
    Iterable,
    Union,
)

from pyvenafi.tpp.api.websdk.enums.config import CodeSignAttributeValues
from pyvenafi.tpp.api.websdk.models.codesign import (
    AppleEnvironment,
    Application,
    ApplicationCollection,
    CertificateEnvironment,
    CSPEnvironment,
    DotNetEnvironment,
    GPGEnvironment,
    Items,
    KeyPairEnvironment,
    Project,
    Rights,
)
from pyvenafi.tpp.api.websdk.models.config import Object
from pyvenafi.tpp.api.websdk.models.identity import Identity
from pyvenafi.tpp.features.bases.feature_base import (
    feature,
    FeatureBase,
)

EnvironmentTypes = Union[
    AppleEnvironment,
    CertificateEnvironment,
    CSPEnvironment,
    DotNetEnvironment,
    GPGEnvironment,
    KeyPairEnvironment,
]

@feature('Project')
class CodeSignProject(FeatureBase):
    def create(
        self,
        name: str,
        owners: Iterable[Union[Identity, str]],
        parent_folder: Union[Object, str] = r'\VED\Code Signing\Projects',
        applications: Iterable[Union[str, Object, Application, ApplicationCollection]] = None,
        auditors: Iterable[Union[Identity, str]] = None,
        description: str = None,
        key_use_approvers: Iterable[Union[Identity, str]] = None,
        key_users: Iterable[Union[Identity, str]] = None,
        environments: Iterable[Union[Object, EnvironmentTypes]] = None,
        raise_if_already_exists: bool = False
    ) -> Project:
        """
        Create a project with the specified values.

        Args:
            name: Name of the project.
            parent_folder: :ref:`config_object` or :ref:`dn` of the parent folder.
            applications: List of :class:`~.models.codesign.Application`\s, :class:`~.models.codesign.ApplicationCollection`\s, :ref:`config_object`\s, or :ref:`dn`\s of application or application collections.
            auditors: List of :ref:`identity_object`\s, :ref:`config_object`\s, or prefixed universal of identities to be project auditors.
            description: Description of the project.
            key_use_approvers: List of :ref:`identity_object`\s, :ref:`config_object`\s, or prefixed universal of identities who can approve environment usage.
            key_users: List of :ref:`identity_object`\s, :ref:`config_object`\s, or prefixed universal of identities who can sign with environments.
            owners: List of :ref:`identity_object`\s, :ref:`config_object`\s, or prefixed universal of identities to be project owners.
            environments: List of signing environments for the project.
            raise_if_already_exists: If the object already exists, raise an error.

        Returns:
            :class:`~.models.codesign.Project`
        """
        parent_dn = self._get_dn(parent_folder)

        applications = [self._get_dn(x) for x in applications] if applications else []
        auditors = [self._get_prefixed_universal(x) for x in auditors] if auditors else []
        key_use_approvers = [self._get_prefixed_universal(x) for x in key_use_approvers] if key_use_approvers else []
        key_users = [self._get_prefixed_universal(x) for x in key_users] if key_users else []
        owners = [self._get_prefixed_universal(x) for x in owners] if owners else []
        dn = rf'{parent_dn}\{name}'
        environments = environments or []

        # Create the project
        try:
            self._api.websdk.Codesign.CreateProject.post(dn=dn)
        except:
            if raise_if_already_exists:
                raise

        project = self.get(dn=dn)

        # Update project attributes
        if any(
            [
                applications,
                auditors,
                description,
                key_use_approvers,
                key_users,
                owners,
                environments,
            ]
        ):
            project.description = description
            project.application_dns = Items(items=applications)
            project.auditors = Items(items=auditors)
            project.key_use_approvers = Items(items=key_use_approvers)
            project.key_users = Items(items=key_users)
            project.owners = Items(items=owners)

            self.update(project=project)

        templates = self._api.websdk.Codesign.EnumerateTemplates.post()

        def get_env_template_and_update_kwargs(
            _e: EnvironmentTypes
        ):
            try:
                if isinstance(_e, AppleEnvironment):
                    _key, _templates = 'apple_environment', templates.apple_templates
                elif isinstance(_e, CertificateEnvironment):
                    _key, _templates = 'certificate_environment', templates.certificate_templates
                elif isinstance(_e, CSPEnvironment):
                    _key, _templates = 'csp_environment', templates.csp_templates
                elif isinstance(_e, DotNetEnvironment):
                    _key, _templates = 'dot_net_environment', templates.dot_net_templates
                elif isinstance(_e, GPGEnvironment):
                    _key, _templates = 'gpg_environment', templates.gpg_templates
                elif isinstance(_e, KeyPairEnvironment):
                    _key, _templates = 'key_pair_environment', templates.key_pair_templates
                else:
                    raise ValueError(f'Invalid type {type(_e)}')
                _template = next(t for t in _templates if t.dn == _e.template_dn)
                return _template, {
                    _key: _e
                }
            except:
                raise ValueError(f'Unable to find template "{_e.template_dn}" for environment "{_e.dn}".')

        for e in environments:
            template, update_kwargs = get_env_template_and_update_kwargs(e)
            self._api.websdk.Codesign.CreateEnvironment.post(
                dn=e.dn,
                project={
                    'Dn'  : project.dn,
                    'Guid': project.guid,
                    'Id'  : project.id
                },
                template=template,
                template_dn=template.dn
            )
            self._api.websdk.Codesign.UpdateEnvironment.post(**update_kwargs)

        return self.get(guid=project.guid)

    def update(
        self,
        project: Project
    ):
        """
        Update a project.

        Args:
            project: The updated :class:`~.models.codesign.Project`.
        """
        self._api.websdk.Codesign.UpdateProject.post(project=project)

    def rename(
        self,
        project: Union[str, Object, Project],
        new_dn: str
    ):
        """
        Rename a project by giving it a new :ref:`dn`.

        Args:
            project: :class:`~.models.codesign.Project`, :ref:`config_object`, or :ref:`dn` of the project.
            new_dn: The new :ref:`dn` for the project.
        """
        dn = self._get_dn(project)
        self._api.websdk.Codesign.RenameProject.post(new_dn=new_dn, dn=dn)

    def get(
        self,
        dn: str = None,
        guid: str = None,
        id: str = None
    ) -> Project:
        """
       Get a project using a :ref:`dn`, :ref:`guid`, or Identifier.

       Args:
           dn: :ref:`dn` of the project object.
           guid: :ref:`guid` of the project object.
           id: Identifier of the project object.

       Returns:
           :class:`~.models.codesign.Project`
       """
        if any([dn, guid, id]):
            output = self._api.websdk.Codesign.GetProject.post(dn=dn, guid=guid, id=id)
        else:
            raise ValueError('You must provide a dn, guid, or id to get a project.')
        if not output.project:
            raise Exception('Project does not exist.')
        return output.project

    def enumerate(
        self,
        _filter: str = None,
        rights: Union[int, Rights] = None
    ) -> list[Project]:
        """
        Enumerate the projects.

        Args:
            _filter: A simple filter on the project name.
            rights: Show only projects for which you have a minimum set of permissions. See :class:`~.models.codesign.Rights`.

        Returns:
            list[:class:`~.models.codesign.Project`]
        """
        if isinstance(rights, Rights):
            rights = rights.value
        output = self._api.websdk.Codesign.EnumerateProjects.post(filter=_filter, rights=rights)
        return output.projects

    def delete(
        self,
        project: Union[str, Object, Project]
    ):
        """
        Delete a project.

        Args:
            project: :class:`~.models.codesign.Project`, :ref:`dn`, or :ref:`config_object` of the project.
        """
        dn = self._get_dn(project)
        if dn:
            self._api.websdk.Codesign.DeleteProject.post(dn=dn)
        else:
            guid = self._get_guid(project)
            if guid:
                self._api.websdk.Codesign.DeleteProject.post(guid=guid)
            else:
                raise ValueError('Invalid argument to delete the project.')

    # region Update Project Status
    def disable(
        self,
        project: Union[Object, Project]
    ):
        """
        Disable a project. Changes project status to "disabled".

        Args:
            project: :class:`~.models.codesign.Project` to disable.
        """
        self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.disabled)

    def submit_for_approval(
        self,
        project: Union[Object, Project]
    ):
        """
        Submit a "draft" project to be approved. Changes project status to "pending".

        Args:
            project: :class:`~.models.codesign.Project` to submit.
        """
        dn = self._get_dn(project)
        project = self.get(dn=dn)
        if project.status == int(CodeSignAttributeValues.ProjectStatus.draft):
            self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.pending)

    def approve(
        self,
        project: Union[Object, Project]
    ):
        """
        Approve a "pending" project. Changes project status to "enabled".

        Args:
            project: :class:`~.models.codesign.Project` to approve.
        """
        dn = self._get_dn(project)
        project = self.get(dn=dn)
        if project.status == int(CodeSignAttributeValues.ProjectStatus.pending):
            self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.enabled)

    def submit_and_approve(
        self,
        project: Union[Object, Project]
    ):
        """
        Approves a "draft" or "pending" project. Changes project status to "enabled".

        Args:
            project: :class:`~.models.codesign.Project` to submit and approve.
        """
        dn = self._get_dn(project)
        project = self.get(dn=dn)
        if project.status == int(CodeSignAttributeValues.ProjectStatus.enabled):
            return
        if project.status == int(CodeSignAttributeValues.ProjectStatus.draft):
            self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.pending)
            time.sleep(5)
            self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.enabled)
        else:
            self._update_status(project=project, project_status=CodeSignAttributeValues.ProjectStatus.enabled)

    def _update_status(
        self,
        project: Union[Object, Project],
        project_status: Union[int, CodeSignAttributeValues.ProjectStatus]
    ):
        project_status = int(project_status)
        dn = self._get_dn(project)
        output = self._api.websdk.Codesign.UpdateProjectStatus.post(project_status=project_status, dn=dn)
        output.assert_valid_response()
    # endregion Update Project Status



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class GeotrustCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Geotrust CA"
    account_type = Attribute('Account Type')
    address = Attribute('Address')
    billing_contact_first_name = Attribute('Billing Contact First Name')
    billing_contact_internet_email_address = Attribute('Billing Contact Internet EMail Address')
    billing_contact_last_name = Attribute('Billing Contact Last Name')
    billing_contact_telephone_number = Attribute('Billing Contact Telephone Number')
    city = Attribute('City')
    country = Attribute('Country')
    interval = Attribute('Interval')
    organization = Attribute('Organization')
    partner_code = Attribute('Partner Code')
    postal_code = Attribute('Postal Code')
    state = Attribute('State')
    technical_contact_address = Attribute('Technical Contact Address')
    technical_contact_city = Attribute('Technical Contact City')
    technical_contact_country = Attribute('Technical Contact Country')
    technical_contact_first_name = Attribute('Technical Contact First Name')
    technical_contact_internet_email_address = Attribute('Technical Contact Internet EMail Address')
    technical_contact_last_name = Attribute('Technical Contact Last Name')
    technical_contact_organization = Attribute('Technical Contact Organization')
    technical_contact_postal_code = Attribute('Technical Contact Postal Code')
    technical_contact_state = Attribute('Technical Contact State')
    technical_contact_telephone_number = Attribute('Technical Contact Telephone Number')
    technical_contact_title = Attribute('Technical Contact Title')
    telephone_number = Attribute('Telephone Number')
    test_account = Attribute('Test Account')
    web_service_url = Attribute('Web Service URL')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class DiscoveryStatisticsAttributes(metaclass=IterableMeta):
    __config_class__ = "Discovery Statistics"
    certificates_found = Attribute('Certificates Found', min_version='21.4')
    completed_scans = Attribute('Completed Scans', min_version='21.4')
    connect_succeeded = Attribute('Connect Succeeded', min_version='21.4')
    keys_found = Attribute('Keys Found', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

# region Models
class Result(ObjectModel):
    code: int = ApiField()

    @property
    def config_result(self) -> str:
        return ResultCodes.Config.get(self.code, 'Unknown')

class AttributeDefinition(ObjectModel):
    name: str = ApiField(alias='Name')
    property: int = ApiField(alias='Property')
    syntax: str = ApiField(alias='Syntax')

class ClassDefinition(ObjectModel):
    containment_names: list[str] = ApiField(alias='ContainmentNames', default_factory=list)
    containment_sub_names: list[str] = ApiField(alias='ContainmentSubNames', default_factory=list)
    mandatory_names: list[str] = ApiField(alias='MandatoryNames', default_factory=list)
    name: str = ApiField(alias='Name')
    naming_names: list[str] = ApiField(alias='NamingNames', default_factory=list)
    optional_names: list[str] = ApiField(alias='OptionalNames', default_factory=list)
    super_class_names: list[str] = ApiField(alias='SuperClassNames', default_factory=list)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes
from pyvenafi.tpp.attributes.connection_base import ConnectionBaseAttributes

class OpenSSLCAAttributes(CertificateAuthorityBaseAttributes, ConnectionBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "OpenSSL CA"
    certificate_directory = Attribute('Certificate Directory', min_version='21.4')
    certificate_file = Attribute('Certificate File', min_version='21.4')
    configuration_file = Attribute('Configuration File', min_version='21.4')
    copy_extensions = Attribute('Copy Extensions', min_version='21.4')
    private_key_file = Attribute('Private Key File', min_version='21.4')
    private_key_password_credential = Attribute('Private Key Password Credential', min_version='21.4')
    temp_directory = Attribute('Temp Directory', min_version='21.4')



from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import identity as ident

class _Identity(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Identity')
        self.AddGroup = self._AddGroup(api_obj=self._api_obj, url=f'{self._url}/AddGroup')
        self.AddGroupMembers = self._AddGroupMembers(api_obj=self._api_obj, url=f'{self._url}/AddGroupMembers')
        self.Browse = self._Browse(api_obj=self._api_obj, url=f'{self._url}/Browse')
        self.GetAssociatedEntries = self._GetAssociatedEntries(
            api_obj=self._api_obj,
            url=f'{self._url}/GetAssociatedEntries'
        )
        self.GetMembers = self._GetMembers(api_obj=self._api_obj, url=f'{self._url}/GetMembers')
        self.GetMemberships = self._GetMemberships(api_obj=self._api_obj, url=f'{self._url}/GetMemberships')
        self.Group = self._Group(api_obj=self._api_obj, url=f'{self._url}/Group')
        self.ReadAttribute = self._ReadAttribute(api_obj=self._api_obj, url=f'{self._url}/ReadAttribute')
        self.RemoveGroupMembers = self._RemoveGroupMembers(api_obj=self._api_obj, url=f'{self._url}/RemoveGroupMembers')
        self.RenameGroup = self._RenameGroup(api_obj=self._api_obj, url=f'{self._url}/RenameGroup')
        self.Self = self._Self(api_obj=self._api_obj, url=f'{self._url}/Self')
        self.SetPassword = self._SetPassword(api_obj=self._api_obj, url=f'{self._url}/SetPassword')
        self.Validate = self._Validate(api_obj=self._api_obj, url=f'{self._url}/Validate')

    class _AddGroup(WebSdkEndpoint):
        def post(self, name: str, members: list[Union[dict, ident.Identity]] = None, products: list[str] = None):
            body = {
                'Name'    : name,
                'Members' : members,
                'Products': products
            }

            class Output(WebSdkOutputModel):
                identity: ident.Identity = ApiField(alias='ID')
                invalid_members: list[ident.InvalidIdentity] = ApiField('InvalidMembers', default_factory=list)
                invalid_owners: list[ident.InvalidIdentity] = ApiField('InvalidOwners', default_factory=list)
                message: str = ApiField(alias='Message')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddGroupMembers(WebSdkEndpoint):
        def put(
            self,
            group: Union[dict, ident.Identity],
            members: list[Union[dict, ident.Identity]],
            show_members: bool = False
        ):
            body = {
                'Group'      : group,
                'Members'    : members,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                invalid_members: list[ident.InvalidIdentity] = ApiField(alias='InvalidMembers', default_factory=list)
                members: list[ident.Identity] = ApiField(alias='Members', default_factory=list)
                message: str = ApiField(alias='Message')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _Browse(WebSdkEndpoint):
        # noinspection ALL
        def post(self, filter: str, limit: int, identity_type: int):
            body = {
                "Filter"      : filter,
                "Limit"       : limit,
                "IdentityType": identity_type
            }

            class Output(WebSdkOutputModel):
                identities: list[ident.Identity] = ApiField('Identities', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetAssociatedEntries(WebSdkEndpoint):
        def post(self, identity: Union[dict, ident.Identity]):
            body = {
                'ID': identity
            }

            class Output(WebSdkOutputModel):
                identities: list[ident.Identity] = ApiField('Identities', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetMembers(WebSdkEndpoint):
        def post(self, identity: Union[dict, ident.Identity], resolve_nested: bool = False):
            body = {
                'ID'           : identity,
                'ResolveNested': int(resolve_nested)
            }

            class Output(WebSdkOutputModel):
                identities: list[ident.Identity] = ApiField(alias='Identities', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetMemberships(WebSdkEndpoint):
        def post(self, identity: Union[dict, ident.Identity]):
            body = {
                'ID': identity
            }

            class Output(WebSdkOutputModel):
                identities: list[ident.Identity] = ApiField(alias='Identities', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Group(WebSdkEndpoint):
        def Prefix(self, prefix: str):
            return self._Prefix(api_obj=self._api_obj, url=f'{self._url}/{prefix}')

        class _Prefix(WebSdkEndpoint):
            def Principal(self, principal: str):
                return self._Principal(api_obj=self._api_obj, url=f'{self._url}/{principal}')

            class _Principal(WebSdkEndpoint):
                def delete(self):
                    class Output(WebSdkOutputModel):
                        message: str = ApiField(alias='Message')

                    return generate_output(output_cls=Output, response=self._delete())

    class _ReadAttribute(WebSdkEndpoint):
        def post(self, attribute_name: str, identity: Union[dict, ident.Identity]):
            body = {
                'ID'           : identity,
                'AttributeName': attribute_name
            }

            class Output(WebSdkOutputModel):
                attributes: list[str] = ApiField(alias='Attributes', default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RemoveGroupMembers(WebSdkEndpoint):
        def put(
            self,
            group: Union[dict, ident.Identity],
            members: list[Union[dict, ident.Identity]],
            show_members: bool = False
        ):
            body = {
                'Group'      : group,
                'Members'    : members,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                invalid_members: list[ident.InvalidIdentity] = ApiField(alias='InvalidMembers', default_factory=list)
                members: list[ident.Identity] = ApiField(alias='Members', default_factory=list)
                message: str = ApiField(alias='Message')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _RenameGroup(WebSdkEndpoint):
        def put(self, group: Union[dict, ident.Identity], new_group_name: str):
            body = {
                'Group'       : group,
                'NewGroupName': new_group_name
            }

            class Output(WebSdkOutputModel):
                identity: ident.Identity = ApiField(alias='ID')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _Self(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                identities: list[ident.Identity] = ApiField(alias='Identities', default_factory=list)

            return generate_output(output_cls=Output, response=self._get())

    class _SetPassword(WebSdkEndpoint):
        def post(self, identity: Union[dict, ident.Identity], password: str, old_password: str = None):
            body = {
                'ID'         : identity,
                'OldPassword': old_password,
                'Password'   : password
            }

            class Output(WebSdkOutputModel):
                identity: ident.Identity = ApiField(alias='ID')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Validate(WebSdkEndpoint):
        def post(self, identity: Union[dict, ident.Identity]):
            body = {
                'ID': identity
            }

            class Output(WebSdkOutputModel):
                identity: ident.Identity = ApiField(alias='ID')

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes

class ReportAssessorAttributes(ReportBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report:Assessor"
    jobdn = Attribute('JobDN')
    options = Attribute('Options')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class JKSAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "JKS"
    backup_store = Attribute('Backup Store', min_version='24.1')
    certificate_label = Attribute('Certificate Label', min_version='21.4')
    create_store = Attribute('Create Store', min_version='21.4')
    disable_ssh_history = Attribute('Disable SSH History', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    java_vendor = Attribute('Java Vendor', min_version='21.4')
    key_algorithm = Attribute('Key Algorithm', min_version='21.4')
    key_store = Attribute('Key Store', min_version='21.4')
    key_store_credential = Attribute('Key Store Credential', min_version='21.4')
    key_store_validation_disabled = Attribute('Key Store Validation Disabled', min_version='21.4')
    keytool_path = Attribute('Keytool Path', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    private_key_location = Attribute('Private Key Location', min_version='21.4')
    protection_type = Attribute('Protection Type', min_version='21.4')
    recycle_alias = Attribute('Recycle Alias', min_version='21.4')
    replace_store = Attribute('Replace Store', min_version='21.4')
    slot_number = Attribute('Slot Number', min_version='21.4')
    softcard_identifier = Attribute('Softcard Identifier', min_version='21.4')
    store_type = Attribute('Store Type', min_version='21.4')
    temp_certificate_label = Attribute('Temp Certificate Label', min_version='21.4')
    use_external_process = Attribute('Use External Process', min_version='23.1')
    version = Attribute('Version', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class Layer7SSGAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Layer 7 SSG"
    certificate_oid = Attribute('Certificate OID')
    certificate_only = Attribute('Certificate Only')
    chain_cert = Attribute('Chain Cert')
    database_validation_disabled = Attribute('Database Validation Disabled')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='15.3')
    first_backup_certificate_oid = Attribute('First Backup Certificate OID')
    first_backup_private_key_oid = Attribute('First Backup Private Key OID')
    https_port = Attribute('HTTPS Port')
    network_validation_disabled = Attribute('Network Validation Disabled')
    outbound_ssl = Attribute('Outbound SSL')
    private_key_oid = Attribute('Private Key OID')
    provision_manner = Attribute('Provision Manner')
    revocation_checking_enabled = Attribute('Revocation Checking Enabled')
    saml_attesting = Attribute('SAML Attesting')
    second_backup_certificate_oid = Attribute('Second Backup Certificate OID')
    second_backup_private_key_oid = Attribute('Second Backup Private Key OID')
    service = Attribute('Service')
    signing_client = Attribute('Signing Client')
    signing_outbound_ssl = Attribute('Signing Outbound SSL')
    signing_saml = Attribute('Signing SAML')
    trust_anchor = Attribute('Trust Anchor')
    verify_hostname = Attribute('Verify Hostname')
    web_credential = Attribute('Web Credential')
    web_service_url = Attribute('Web Service URL')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class BasicAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Basic"
    certificate_file = Attribute('Certificate File', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes
from pyvenafi.tpp.attributes.top import TopAttributes

class CodeSigningKeyTimeConstraintAttributes(ScheduleBaseAttributes, TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Key Time Constraint"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.flow_action_base import FlowActionBaseAttributes

class FlowEventActionAttributes(FlowActionBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Flow Event Action"
    condition = Attribute('Condition', min_version='21.4')
    data_content = Attribute('Data Content', min_version='21.4')
    event = Attribute('Event', min_version='21.4')
    severity = Attribute('Severity', min_version='21.4')
    text1_content = Attribute('Text1 Content', min_version='21.4')
    text2_content = Attribute('Text2 Content', min_version='21.4')
    value1_content = Attribute('Value1 Content', min_version='21.4')
    value2_content = Attribute('Value2 Content', min_version='21.4')



from __future__ import annotations

import time
from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import (
    credential,
    identity,
)

class _Credentials(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Credentials')
        self.Adaptable = self._Adaptable(api_obj=api_obj, url=f'{self._url}/Adaptable')
        self.Connector = self._Connector(api_obj=api_obj, url=f'{self._url}/Connector')
        self.Create = self._Create(api_obj=api_obj, url=f'{self._url}/Create')
        self.Delete = self._Delete(api_obj=api_obj, url=f'{self._url}/Delete')
        self.Enumerate = self._Enumerate(api_obj=api_obj, url=f'{self._url}/Enumerate')
        self.Rename = self._Rename(api_obj=api_obj, url=f'{self._url}/Rename')
        self.Retrieve = self._Retrieve(api_obj=api_obj, url=f'{self._url}/Retrieve')
        self.Update = self._Update(api_obj=api_obj, url=f'{self._url}/Update')
        self.CyberArk = self._CyberArk(api_obj=api_obj, url=f'{self._url}/CyberArk')

    class _Adaptable(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.Create = self._Create(api_obj=self._api_obj, url=f'{self._url}/Create')
            self.Update = self._Update(api_obj=self._api_obj, url=f'{self._url}/Update')

        class _Create(WebSdkEndpoint):
            def post(
                self, credential_path: str, credential_type: credential.CredentialType, connector_name: str,
                custom_fields: list[dict[str, str]]
            ):
                body = {
                    'CredentialPath': credential_path,
                    'CredentialType': credential_type,
                    'ConnectorName' : connector_name,
                    'CustomFields'  : custom_fields
                }

                class Output(WebSdkOutputModel):
                    result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Update(WebSdkEndpoint):
            def post(
                self, credential_path: str, credential_type: str, connector_name: str,
                custom_fields: list[dict[str, str]]
            ):
                body = {
                    'CredentialPath': credential_path,
                    'CredentialType': credential_type,
                    'ConnectorName' : connector_name,
                    'CustomFields'  : custom_fields
                }

                class Output(WebSdkOutputModel):
                    result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data=body))

    class _Connector(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.Adaptable = self._Adaptable(api_obj=self._api_obj, url=f'{self._url}/Adaptable')

        class _Adaptable(WebSdkEndpoint):
            def post(
                self, connector_name: str, powershell_script: str, service_address: str,
                service_credential: str, allowed_identities: list[str] = None,
                description: str = None
            ):
                body = {
                    'AllowedIdentities': allowed_identities,
                    'ConnectorName'    : connector_name,
                    'Description'      : description,
                    'PowershellScript' : powershell_script,
                    'ServiceAddress'   : service_address,
                    'ServiceCredential': service_credential
                }

                class Output(WebSdkOutputModel):
                    success: bool = ApiField(alias='Success')

                return generate_output(output_cls=Output, response=self._post(data=body))

            def Guid(self, guid: str):
                return self._Guid(api_obj=self._api_obj, url=f'{self._url}/{guid}')

            class _Guid(WebSdkEndpoint):
                def delete(self):
                    class Output(WebSdkOutputModel):
                        success: bool = ApiField(alias='Success')

                    return generate_output(output_cls=Output, response=self._delete())

                def get(self):
                    class Output(WebSdkOutputModel):
                        allowed_identities: list[str] = ApiField(default_factory=list, alias='AllowedIdentities')
                        powershell_script: str = ApiField(alias='PowershellScript')
                        service_address: str = ApiField(alias='ServiceAddress')
                        service_credential: str = ApiField(alias='ServiceCredential')
                        success: bool = ApiField(alias='Success')

                    return generate_output(output_cls=Output, response=self._get())

                def put(
                    self, connector_name: str = None, powershell_script: str = None,
                    service_address: str = None, service_credential: str = None,
                    allowed_identities: list[str] = None, description: str = None
                ):
                    body = {
                        'AllowedIdentities': allowed_identities,
                        'ConnectorName'    : connector_name,
                        'Description'      : description,
                        'PowershellScript' : powershell_script,
                        'ServiceAddress'   : service_address,
                        'ServiceCredential': service_credential
                    }

                    class Output(WebSdkOutputModel):
                        success: bool = ApiField(alias='Success')

                    return generate_output(output_cls=Output, response=self._put(data=body))

    class _Create(WebSdkEndpoint):
        def post(
            self,
            credential_path: str,
            friendly_name: str,
            values: list[credential.NameTypeValue],
            password: str = None,
            description: str = None,
            encryption_key: str = None,
            shared: bool = False,
            expiration: int = None,
            contact: list[str] = None
        ):
            body = {
                'CredentialPath': credential_path,
                'Password'      : password,
                'FriendlyName'  : friendly_name,
                'Values'        : values,
                'Expiration'    : f'/Date({expiration})/',
                'Description'   : description,
                'EncryptionKey' : encryption_key,
                'Shared'        : shared,
                'Contact'       : contact
            }

            class Output(WebSdkOutputModel):
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Delete(WebSdkEndpoint):
        def post(self, credential_path: str):
            body = {
                'CredentialPath': credential_path
            }

            class Output(WebSdkOutputModel):
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Enumerate(WebSdkEndpoint):
        def post(self, credential_path: str, pattern: str = None, recursive: bool = False):
            body = {
                'CredentialPath': credential_path,
                'Recursive'     : recursive,
                'Pattern'       : pattern
            }

            class Output(WebSdkOutputModel):
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))
                credential_infos: list[credential.CredentialInfo] = ApiField(
                    default_factory=list,
                    alias='CredentialInfos'
                )

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Rename(WebSdkEndpoint):
        def post(self, credential_path: str, new_credential_path: str):
            body = {
                'CredentialPath'   : credential_path,
                'NewCredentialPath': new_credential_path
            }

            class Output(WebSdkOutputModel):
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Retrieve(WebSdkEndpoint):
        def post(self, credential_path: str):
            body = {
                'CredentialPath': credential_path
            }

            class Output(WebSdkOutputModel):
                classname: str = ApiField(alias='Classname')
                contact: list[identity.Identity] = ApiField(alias='Contact', default_factory=list)
                description: str = ApiField(alias='Description')
                expiration: datetime = ApiField(alias='Expiration')
                friendly_name: str = ApiField(alias='FriendlyName')
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))
                values: list[credential.NameTypeValue] = ApiField(default_factory=list, alias='Values')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Update(WebSdkEndpoint):
        def post(
            self,
            credential_path: str,
            friendly_name: str,
            values: list[credential.NameTypeValue],
            description: str = None,
            encryption_key: str = None,
            shared: bool = False,
            expiration: int = None,
            contact: list[str] = None
        ):
            body = {
                'CredentialPath': credential_path,
                'FriendlyName'  : friendly_name,
                'Values'        : values,
                'Description'   : description,
                'EncryptionKey' : encryption_key,
                'Shared'        : shared,
                'Contact'       : contact
            }

            if expiration:
                exp_date = expiration
            else:
                # Expire in 10 years.
                exp_date = int((time.time() + (60 * 60 * 24 * 365 * 10)) * 1000)

            body.update(
                {
                    'Expiration': r'/Date(%s)/' % exp_date
                }
            )

            class Output(WebSdkOutputModel):
                result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CyberArk(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.Create = self._Create(api_obj=self._api_obj, url=f'{self._url}/Create')
            self.Update = self._Update(api_obj=self._api_obj, url=f'{self._url}/Update')

        class _Create(WebSdkEndpoint):
            def post(
                self, cyber_ark_username: str, cyber_ark_password: str, username: str, app_id: str, safe_name: str,
                folder_name: str, account_name: str, credential_path: str
            ):
                body = {
                    'CyberArkUsername': cyber_ark_username,
                    'CyberArkPassword': cyber_ark_password,
                    'Username'        : username,
                    'AppID'           : app_id,
                    'SafeName'        : safe_name,
                    'FolderName'      : folder_name,
                    'AccountName'     : account_name,
                    'CredentialPath'  : credential_path
                }

                class Output(WebSdkOutputModel):
                    result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data=body))

        class _Update(WebSdkEndpoint):
            def post(
                self, cyber_ark_username: str, cyber_ark_password: str, username: str, app_id: str, safe_name: str,
                folder_name: str, account_name: str, credential_path: str
            ):
                body = {
                    'CyberArkUsername': cyber_ark_username,
                    'CyberArkPassword': cyber_ark_password,
                    'Username'        : username,
                    'AppID'           : app_id,
                    'SafeName'        : safe_name,
                    'FolderName'      : folder_name,
                    'AccountName'     : account_name,
                    'CredentialPath'  : credential_path
                }

                class Output(WebSdkOutputModel):
                    result: credential.Result = ApiField(alias='Result', converter=lambda x: credential.Result(code=x))

                return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class MonitoringBaseAttributes(metaclass=IterableMeta):
    __config_class__ = "Monitoring Base"
    creation_date = Attribute('Creation Date', min_version='21.4')
    escalation_notice_interval = Attribute('Escalation Notice Interval', min_version='21.4')
    escalation_notice_start = Attribute('Escalation Notice Start', min_version='21.4')
    expiration_notice_interval = Attribute('Expiration Notice Interval', min_version='21.4')
    expiration_notice_start = Attribute('Expiration Notice Start', min_version='21.4')
    last_notification = Attribute('Last Notification', min_version='21.4')
    notes = Attribute('Notes', min_version='21.4')
    notification_disabled = Attribute('Notification Disabled', min_version='21.4')
    notified_on = Attribute('Notified On', min_version='24.1')
    validity_period = Attribute('Validity Period', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class EntrustPKIGatewayAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Entrust PKI Gateway"
    ca_capabilities = Attribute('CA Capabilities', min_version='21.4')
    ca_name = Attribute('CA Name', min_version='21.4')
    exclude_subjectvariables = Attribute('Exclude SubjectVariables', min_version='21.4')
    profile_capabilities = Attribute('Profile Capabilities', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import connectors_service

class _connectors_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.connectors = self._connectors(api_obj=self._api_obj, url=f'{self._url}/connectors')

        class _connectors(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self):
                class Output(CloudApiOutputModel):
                    ConnectorsResponse: connectors_service.ConnectorsResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    rc_mapping={
                        200: 'ConnectorsResponse'
                    }
                )

            def post(self, ConnectorsCreationRequest: connectors_service.ConnectorsCreationRequest):
                data = {**ConnectorsCreationRequest.dict()}

                class Output(CloudApiOutputModel):
                    ConnectorsInformation: connectors_service.ConnectorsInformation

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        201: 'ConnectorsInformation'
                    }
                )

            class _ID(CloudApiEndpoint):
                def delete(self):
                    class Output(CloudApiOutputModel):
                        pass

                    return generate_output(output_cls=Output, response=self._delete(params={}))

                def get(self):
                    class Output(CloudApiOutputModel):
                        ConnectorsInformation: connectors_service.ConnectorsInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'ConnectorsInformation'
                        }
                    )

                def put(self, ConnectorsUpdateRequest: connectors_service.ConnectorsUpdateRequest):
                    data = {**ConnectorsUpdateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        ConnectorsInformation: connectors_service.ConnectorsInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._put(data=data),
                        rc_mapping={
                            200: 'ConnectorsInformation'
                        }
                    )



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from datetime import datetime
from typing import (
    Any,
    List,
    Literal,
)
from uuid import UUID

AnyValue = Any

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class Expression(ObjectModel):
    ...

class JsonNode(ObjectModel):
    ...

class MachineBatchProvisioningResultInformation(ObjectModel):
    endDate: datetime = ApiField(alias='endDate')
    errorMessage: str = ApiField(alias='errorMessage')
    machineIdentitiesCount: int = ApiField(alias='machineIdentitiesCount')
    machineIdentitiesFailedCount: int = ApiField(alias='machineIdentitiesFailedCount')
    machineIdentitiesSucceedCount: int = ApiField(alias='machineIdentitiesSucceedCount')
    startDate: datetime = ApiField(alias='startDate')
    type: Literal['MANUAL', 'SCHEDULED'] = ApiField(alias='type')

class MachineCreationRequest(ObjectModel):
    connectionDetails: AnyValue = ApiField(alias='connectionDetails')
    dekId: str = ApiField(alias='dekId')
    edgeInstanceId: UUID = ApiField(alias='edgeInstanceId')
    machineTypeId: UUID = ApiField(alias='machineTypeId')
    name: str = ApiField(alias='name')
    owningTeamId: UUID = ApiField(alias='owningTeamId')
    pluginId: UUID = ApiField(alias='pluginId')
    status: Literal['DELETING', 'DRAFT', 'UNVERIFIED', 'VERIFIED'] = ApiField(alias='status')
    tags: List[str] = ApiField(alias='tags', default_factory=list)

class MachineDiscoveryResultInformation(ObjectModel):
    certificatesCountCurrent: int = ApiField(alias='certificatesCountCurrent')
    certificatesCountTotal: int = ApiField(alias='certificatesCountTotal')
    discoveryStatus: Literal['ABORTED', 'ABORTING', 'COMPLETED', 'FAILED',
    'INITIATED', 'NEVER INITIATED', 'RUNNING'] = ApiField(alias='discoveryStatus')
    endDate: datetime = ApiField(alias='endDate')
    errorCount: int = ApiField(alias='errorCount')
    machineIdentitiesCount: int = ApiField(alias='machineIdentitiesCount')
    machineIdentitiesDeletedCount: int = ApiField(alias='machineIdentitiesDeletedCount')
    machineIdentitiesMissingCount: int = ApiField(alias='machineIdentitiesMissingCount')
    startDate: datetime = ApiField(alias='startDate')

class MachineDocumentInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    integrationId: UUID = ApiField(alias='integrationId')
    machineIdentitiesCount: int = ApiField(alias='machineIdentitiesCount')
    machineName: str = ApiField(alias='machineName')
    machineType: str = ApiField(alias='machineType')
    machineTypeId: UUID = ApiField(alias='machineTypeId')
    modificationDate: datetime = ApiField(alias='modificationDate')
    ownership: MachineOwnership = ApiField(alias='ownership')
    owningTeam: UUID = ApiField(alias='owningTeam')
    pluginId: UUID = ApiField(alias='pluginId')
    pluginName: str = ApiField(alias='pluginName')
    status: Literal['DELETING', 'DRAFT', 'UNVERIFIED', 'VERIFIED'] = ApiField(alias='status')

class MachineDocumentResponse(ObjectModel):
    machines: List[MachineDocumentInformation] = ApiField(alias='machines', default_factory=list)
    totalCount: int = ApiField(alias='totalCount')

class MachineIdentityCreationRequest(ObjectModel):
    binding: AnyValue = ApiField(alias='binding')
    certificateId: UUID = ApiField(alias='certificateId')
    keystore: AnyValue = ApiField(alias='keystore')
    machineId: UUID = ApiField(alias='machineId')

class MachineIdentityDocumentInformation(ObjectModel):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)
    certificateFingerprint: str = ApiField(alias='certificateFingerprint')
    certificateId: UUID = ApiField(alias='certificateId')
    certificateName: str = ApiField(alias='certificateName')
    certificateValidityEnd: datetime = ApiField(alias='certificateValidityEnd')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    lastSeenOn: datetime = ApiField(alias='lastSeenOn')
    machineId: UUID = ApiField(alias='machineId')
    machineName: str = ApiField(alias='machineName')
    modificationDate: datetime = ApiField(alias='modificationDate')
    status: Literal['DISCOVERED', 'FAILED', 'INSTALLED', 'MISSING', 'NEW', 'PENDING', 'VALIDATED'] = ApiField(
        alias='status'
    )

class MachineIdentityDocumentResponse(ObjectModel):
    machineIdentities: List[MachineIdentityDocumentInformation] = ApiField(
        alias='machineIdentities',
        default_factory=list
    )

class MachineIdentityInformation(ObjectModel):
    binding: AnyValue = ApiField(alias='binding')
    certificateId: UUID = ApiField(alias='certificateId')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    keystore: AnyValue = ApiField(alias='keystore')
    lastSeenOn: datetime = ApiField(alias='lastSeenOn')
    machineId: UUID = ApiField(alias='machineId')
    metadata: AnyValue = ApiField(alias='metadata')
    modificationDate: datetime = ApiField(alias='modificationDate')
    status: Literal['DISCOVERED', 'FAILED', 'INSTALLED', 'MISSING', 'NEW', 'PENDING', 'VALIDATED'] = ApiField(
        alias='status'
    )

class MachineIdentityResponse(ObjectModel):
    machineIdentities: List[MachineIdentityInformation] = ApiField(alias='machineIdentities', default_factory=list)

class MachineIdentitySearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class MachineIdentityUpdateRequest(ObjectModel):
    binding: AnyValue = ApiField(alias='binding')
    certificateId: UUID = ApiField(alias='certificateId')
    keystore: AnyValue = ApiField(alias='keystore')
    status: Literal['DISCOVERED', 'FAILED', 'INSTALLED', 'MISSING', 'NEW', 'PENDING', 'VALIDATED'] = ApiField(
        alias='status'
    )

class MachineIdentityWorkflowInformation(ObjectModel):
    workflowId: str = ApiField(alias='workflowId')
    workflowName: str = ApiField(alias='workflowName')

class MachineIdentityWorkflowRequest(ObjectModel):
    workflowInput: ProvisionCertificateWorkflowInputInformation = ApiField(alias='workflowInput')
    workflowName: str = ApiField(alias='workflowName')

class MachineInformation(ObjectModel):
    batchProvisioningResult: MachineBatchProvisioningResultInformation = ApiField(alias='batchProvisioningResult')
    batchProvisioningSchedulerEnabled: bool = ApiField(alias='batchProvisioningSchedulerEnabled')
    batchProvisioningSchedulerPattern: SchedulerPatternInformation = ApiField(alias='batchProvisioningSchedulerPattern')
    batchProvisioningStatus: Literal['ABORTED', 'ABORTING', 'COMPLETED', 'FAILED',
    'INITIATED', 'NEVER_INITIATED', 'RUNNING'] = ApiField(alias='batchProvisioningStatus')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    discoveryJson: AnyValue = ApiField(alias='discoveryJson')
    discoverySchedulerEnabled: bool = ApiField(alias='discoverySchedulerEnabled')
    discoverySchedulerPattern: SchedulerPatternInformation = ApiField(alias='discoverySchedulerPattern')
    discoveryStatus: Literal['ABORTED', 'ABORTING', 'COMPLETED', 'FAILED',
    'INITIATED', 'NEVER INITIATED', 'RUNNING'] = ApiField(alias='discoveryStatus')
    edgeInstanceId: UUID = ApiField(alias='edgeInstanceId')
    id: UUID = ApiField(alias='id')
    integrationId: UUID = ApiField(alias='integrationId')
    machineType: str = ApiField(alias='machineType')
    modificationDate: datetime = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    owningTeamId: UUID = ApiField(alias='owningTeamId')
    pluginId: UUID = ApiField(alias='pluginId')
    status: Literal['DELETING', 'DRAFT', 'UNVERIFIED', 'VERIFIED'] = ApiField(alias='status')

class MachineOwnership(ObjectModel):
    owningTeams: List[UUID] = ApiField(alias='owningTeams', default_factory=list)

class MachineTypeInformation(ObjectModel):
    id: UUID = ApiField(alias='id')
    machineType: str = ApiField(alias='machineType')
    pluginId: UUID = ApiField(alias='pluginId')

class MachineTypeResponse(ObjectModel):
    machineTypes: List[MachineTypeInformation] = ApiField(alias='machineTypes', default_factory=list)

class MachineUpdateRequest(ObjectModel):
    batchProvisioningSchedulerEnabled: bool = ApiField(alias='batchProvisioningSchedulerEnabled')
    batchProvisioningSchedulerPattern: SchedulerPatternInformation = ApiField(alias='batchProvisioningSchedulerPattern')
    connectionDetails: AnyValue = ApiField(alias='connectionDetails')
    discoveryJson: AnyValue = ApiField(alias='discoveryJson')
    discoverySchedulerEnabled: bool = ApiField(alias='discoverySchedulerEnabled')
    discoverySchedulerPattern: SchedulerPatternInformation = ApiField(alias='discoverySchedulerPattern')
    edgeInstanceId: UUID = ApiField(alias='edgeInstanceId')
    name: str = ApiField(alias='name')
    owningTeamId: UUID = ApiField(alias='owningTeamId')
    status: Literal['DELETING', 'DRAFT', 'UNVERIFIED', 'VERIFIED'] = ApiField(alias='status')

class MachineWorkflowRequest(ObjectModel):
    workflowInput: AnyValue = ApiField(alias='workflowInput')
    workflowName: str = ApiField(alias='workflowName')

class MachinesResponse(ObjectModel):
    machines: List[MachineInformation] = ApiField(alias='machines', default_factory=list)

class MachinesSearchRequest(ObjectModel):
    expression: Expression = ApiField(alias='expression')
    ordering: Ordering = ApiField(alias='ordering')
    paging: Paging = ApiField(alias='paging')

class OrderObject(ObjectModel):
    direction: Literal['ASC', 'DESC'] = ApiField(alias='direction')
    field: str = ApiField(alias='field')

class Ordering(ObjectModel):
    orders: List[OrderObject] = ApiField(alias='orders', default_factory=list)

class Paging(ObjectModel):
    pageNumber: int = ApiField(alias='pageNumber')
    pageSize: int = ApiField(alias='pageSize')

class ProvisionCertificateWorkflowInputInformation(ObjectModel):
    wsClientId: str = ApiField(alias='wsClientId')

class SchedulerPatternInformation(ObjectModel):
    recurrenceType: str = ApiField(alias='recurrenceType')

class WeeklyPatternInformation(SchedulerPatternInformation):
    daysOfWeek: List[Literal['FRIDAY', 'MONDAY', 'SATURDAY', 'SUNDAY', 'THURSDAY',
    'TUESDAY', 'WEDNESDAY']] = ApiField(alias='daysOfWeek', default_factory=list)
    recurrenceTime: datetime = ApiField(alias='recurrenceTime')

class CronPatternInformation(SchedulerPatternInformation):
    cronExpression: str = ApiField(alias='cronExpression')

class MonthlyPatternInformation(SchedulerPatternInformation):
    daysOfMonth: List[int] = ApiField(alias='daysOfMonth', default_factory=list)
    recurrenceTime: datetime = ApiField(alias='recurrenceTime')

CronPatternInformation.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
Expression.update_forward_refs()
JsonNode.update_forward_refs()
MachineBatchProvisioningResultInformation.update_forward_refs()
MachineCreationRequest.update_forward_refs()
MachineDiscoveryResultInformation.update_forward_refs()
MachineDocumentInformation.update_forward_refs()
MachineDocumentResponse.update_forward_refs()
MachineIdentityCreationRequest.update_forward_refs()
MachineIdentityDocumentInformation.update_forward_refs()
MachineIdentityDocumentResponse.update_forward_refs()
MachineIdentityInformation.update_forward_refs()
MachineIdentityResponse.update_forward_refs()
MachineIdentitySearchRequest.update_forward_refs()
MachineIdentityUpdateRequest.update_forward_refs()
MachineIdentityWorkflowInformation.update_forward_refs()
MachineIdentityWorkflowRequest.update_forward_refs()
MachineInformation.update_forward_refs()
MachineOwnership.update_forward_refs()
MachineTypeInformation.update_forward_refs()
MachineTypeResponse.update_forward_refs()
MachineUpdateRequest.update_forward_refs()
MachineWorkflowRequest.update_forward_refs()
MachinesResponse.update_forward_refs()
MachinesSearchRequest.update_forward_refs()
MonthlyPatternInformation.update_forward_refs()
OrderObject.update_forward_refs()
Ordering.update_forward_refs()
Paging.update_forward_refs()
ProvisionCertificateWorkflowInputInformation.update_forward_refs()
SchedulerPatternInformation.update_forward_refs()
WeeklyPatternInformation.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.metadata_base import MetadataBaseAttributes

class MetadataTextAttributes(MetadataBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Metadata Text"
    allowed_characters = Attribute('Allowed Characters', min_version='21.4')
    error_message = Attribute('Error Message', min_version='21.4')
    mask = Attribute('Mask', min_version='21.4')
    maximum_length = Attribute('Maximum Length', min_version='21.4')
    minimum_length = Attribute('Minimum Length', min_version='21.4')
    regular_expression = Attribute('Regular Expression', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class UniCERTCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "UniCERT CA"
    ca_dn = Attribute('CA DN')
    ra_dn = Attribute('RA DN')
    secure = Attribute('Secure')
    web_instance = Attribute('Web Instance')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class AgentBaseAttributes(metaclass=IterableMeta):
    __config_class__ = "Agent Base"
    agent_guid = Attribute('Agent GUID', min_version='21.4')
    location = Attribute('Location', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.ssh_key import SSHKeyAttributes

class SSHUserKeyAttributes(SSHKeyAttributes, metaclass=IterableMeta):
    __config_class__ = "SSH User Key"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.log_channel import LogChannelAttributes

class LogSMTPAttributes(LogChannelAttributes, metaclass=IterableMeta):
    __config_class__ = "Log SMTP"
    attachment_behavior = Attribute('Attachment Behavior', min_version='21.4')
    bcc = Attribute('BCC', min_version='22.2')
    cc = Attribute('CC', min_version='21.4')
    component_vault_id = Attribute('Component Vault Id', min_version='23.3')
    factory_vault_id = Attribute('Factory Vault Id', min_version='23.3')
    host = Attribute('Host', min_version='21.4')
    item_vault_id = Attribute('Item Vault Id', min_version='21.4')
    log_delivery = Attribute('Log Delivery', min_version='21.4')
    message_body = Attribute('Message Body', min_version='21.4')
    no_html = Attribute('No Html', min_version='23.3')
    recipient = Attribute('Recipient', min_version='21.4')
    smtp_credentials = Attribute('SMTP Credentials', min_version='21.4')
    secure = Attribute('Secure', min_version='21.4')
    sender = Attribute('Sender', min_version='21.4')
    subject = Attribute('Subject', min_version='21.4')
    template_vault_id = Attribute('Template Vault Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class PKCS12Attributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "PKCS#12"
    bundle_certificate = Attribute('Bundle Certificate', min_version='21.4')
    certificate_chain_file = Attribute('Certificate Chain File', min_version='21.4')
    certificate_file = Attribute('Certificate File', min_version='21.4')
    create_store = Attribute('Create Store', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    friendly_name = Attribute('Friendly Name', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    recycle_alias = Attribute('Recycle Alias', min_version='21.4')
    replace_store = Attribute('Replace Store', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class ApacheAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Apache"
    application_id = Attribute('Application ID', min_version='21.4')
    certificate_chain_file = Attribute('Certificate Chain File', min_version='21.4')
    certificate_file = Attribute('Certificate File', min_version='21.4')
    client_tools_path = Attribute('Client Tools Path', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    kmdata_key_file = Attribute('KMDATA Key File', min_version='22.2')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    ocs_identifier = Attribute('OCS Identifier', min_version='21.4')
    overwrite_existing_chain = Attribute('Overwrite Existing Chain', min_version='21.4')
    pbes2_algorithm = Attribute('PBES2 Algorithm', min_version='21.4')
    partition_password_credential = Attribute('Partition Password Credential', min_version='21.4')
    private_key_file = Attribute('Private Key File', min_version='21.4')
    private_key_label = Attribute('Private Key Label', min_version='21.4')
    private_key_location = Attribute('Private Key Location', min_version='21.4')
    protection_type = Attribute('Protection Type', min_version='21.4')
    slot_number = Attribute('Slot Number', min_version='21.4')
    softcard_identifier = Attribute('Softcard Identifier', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class LicenseRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "License Root"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class JSSNamespaceAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "JSS Namespace"
    contact = Attribute('Contact', min_version='22.4')
    description = Attribute('Description', min_version='22.4')
    disabled = Attribute('Disabled', min_version='22.4')
    discovered_by_dn = Attribute('Discovered By DN', min_version='22.4')
    jss_namespace_name = Attribute('JSS Namespace Name', min_version='22.4')
    last_sync = Attribute('Last Sync', min_version='22.4')



from __future__ import annotations

class SubSystemTypes:
    aperture = 'Aperture'
    client = 'Client'
    config = 'Config'
    rights = 'Rights'
    secret_store_config = 'SecretStoreconfig'
    websdk = 'WebSDK'



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class LicenseAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "License"
    license_key = Attribute('License Key')



from uuid import UUID
from typing import Union
from pyvenafi.cloud.api.cloud_api import CloudApi

class Authenticate:
    """
    Authenticates to Venafi Cloud API.
    """

    def __init__(self, server: str, api_key: Union[UUID, str]):
        self.cloud_api = CloudApi(server=server, api_key=api_key)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes

class ReportBaseAttributes(DriverBaseAttributes, ScheduleBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report Base"
    csv_vault_id = Attribute('CSV Vault Id')
    configuration = Attribute('Configuration')
    creation_date = Attribute('Creation Date')
    credential = Attribute('Credential')
    delivery_type = Attribute('Delivery Type')
    format = Attribute('Format')
    html_vault_id = Attribute('HTML Vault Id')
    internet_email_address = Attribute('Internet EMail Address')
    last_run = Attribute('Last Run')
    options = Attribute('Options')
    pdf_vault_id = Attribute('PDF Vault Id')
    publishing_host = Attribute('Publishing Host')
    publishing_location = Attribute('Publishing Location')
    rtf_vault_id = Attribute('RTF Vault Id')
    report_template = Attribute('Report Template')
    skip_empty = Attribute('Skip Empty')
    title = Attribute('Title')
    xml_vault_id = Attribute('XML Vault Id')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.client_work_base import ClientWorkBaseAttributes

class SSHWorkAttributes(ClientWorkBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "SSH Work"
    days_of_month = Attribute('Days Of Month', min_version='21.4')
    days_of_week = Attribute('Days Of Week', min_version='21.4')
    exclude_remote_mount_points = Attribute('Exclude Remote Mount Points', min_version='21.4')
    interval = Attribute('Interval', min_version='21.4')
    max_filesize = Attribute('Max Filesize', min_version='21.4')
    new_device_object_landing = Attribute('New Device Object Landing', min_version='21.4')
    remediation_interval = Attribute('Remediation Interval', min_version='21.4')
    remediation_schedule_type = Attribute('Remediation Schedule Type', min_version='21.4')
    remediation_start_time = Attribute('Remediation Start Time', min_version='21.4')
    ssh_scanner_service_path = Attribute('SSH Scanner Service Path', min_version='21.4')
    ssh_scanner_user_path = Attribute('SSH Scanner User Path', min_version='21.4')
    schedule_type = Attribute('Schedule Type', min_version='21.4')
    server_path_defaults_disabled = Attribute('Server Path Defaults Disabled', min_version='21.4')
    start_time = Attribute('Start Time', min_version='21.4')
    user_path_defaults_disabled = Attribute('User Path Defaults Disabled', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class HydrantIdCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "HydrantId CA"
    api_credentials = Attribute('API Credentials', min_version='21.4')
    account_name = Attribute('Account Name', min_version='21.4')
    account_organization = Attribute('Account Organization', min_version='21.4')
    certificate_type = Attribute('Certificate Type', min_version='21.4')
    subscriber_email = Attribute('Subscriber Email', min_version='21.4')
    ui_credentials = Attribute('UI Credentials', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')
    web_ui_url = Attribute('Web UI URL', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.organization import OrganizationAttributes

class IdentityRootAttributes(OrganizationAttributes, metaclass=IterableMeta):
    __config_class__ = "Identity Root"
    authentication_scheme = Attribute('Authentication Scheme', min_version='21.4')
    identity_cache_timeout = Attribute('Identity Cache Timeout', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.flow_action_base import FlowActionBaseAttributes

class FlowActionApproverBaseAttributes(FlowActionBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Flow Action Approver Base"
    minimum_approvers = Attribute('Minimum Approvers', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.credential_base import CredentialBaseAttributes

class GenericCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Generic Credential"



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

class Result(ObjectModel):
    code: int = ApiField()

    @property
    def flow_result(self) -> str:
        return ResultCodes.Flow.get(self.code, 'Unknown')

class Approval(ObjectModel):
    approval_time: datetime = ApiField(alias='ApprovalTime')
    universal: str = ApiField(alias='Universal')

class KeyValue(ObjectModel):
    key: str = ApiField(alias='Key')
    value: str = ApiField(alias='Value')

class Ticket(ObjectModel):
    approvals: list[Approval] = ApiField(alias='Approvals', default_factory=list)
    approvers: list[str] = ApiField(alias='Approvers', default_factory=list)
    creation_time: datetime = ApiField(alias='CreationTime')
    environment: list[KeyValue] = ApiField(alias='Environment', default_factory=list)
    flow_process_id: int = ApiField(alias='FlowProcessId')
    id: int = ApiField(alias='Id')
    identifier: str = ApiField(alias='Identifier')
    product_code: int = ApiField(alias='ProductCode')
    remaining_uses: int = ApiField(alias='RemainingUses')
    required_approvals: int = ApiField(alias='RequiredApprovals')



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

class SshWebResponse(ObjectModel):
    success: bool = ApiField(alias='Success')
    error_code: int = ApiField(alias='ErrorCode')
    error_message: str = ApiField(alias='ErrorMessage')

class ConnectionResult(ObjectModel):
    device_guid: str = ApiField(alias='DeviceGuid')
    error: str = ApiField(alias='Error')
    success: bool = ApiField(alias='Success')

class DeviceData(ObjectModel):
    dn: str = ApiField(alias='Dn')
    device_guid: str = ApiField(alias='DeviceGuid')
    host_name: str = ApiField(alias='HostName')
    is_compliant: bool = ApiField(alias='IsCompliant')
    type: str = ApiField(alias='Type')

class KeyData(ObjectModel):
    active_from: datetime = ApiField(alias='ActiveFrom')
    algorithm: str = ApiField(alias='Algorithm')
    allowed_source_restriction: list[str] = ApiField(alias='AllowedSourceRestriction', default_factory=list)
    approver: list[str] = ApiField(alias='Approver', default_factory=list)
    comment: str = ApiField(alias='Comment')
    denied_source_restriction: list[str] = ApiField(alias='DeniedSourceRestriction', default_factory=list)
    device_guid: str = ApiField(alias='DeviceGuid')
    filepath: str = ApiField(alias='Filepath')
    fingerprint_md5: str = ApiField(alias='FingerprintMD5')
    fingerprint_sha256: str = ApiField(alias='FingerprintSHA256')
    force_command: str = ApiField(alias='ForceCommand')
    format: str = ApiField(alias='Format')
    is_encrypted: bool = ApiField(alias='IsEncrypted')
    key_id: int = ApiField(alias='KeyId')
    keysetid: str = ApiField(alias='KeysetId')
    last_used: datetime = ApiField(alias='LastUsed')
    length: int = ApiField(alias='Length')
    notes: str = ApiField(alias='Notes')
    options: list[str] = ApiField(alias='Options', default_factory=list)
    process_error: str = ApiField(alias='ProcessError')
    process_status: str = ApiField(alias='ProcessStatus')
    reason: str = ApiField(alias='Reason')
    rotation_stage: int = ApiField(alias='RotationStage')
    type: str = ApiField(alias='Type')
    username: str = ApiField(alias='Username')
    violation_status: list[int] = ApiField(alias='ViolationStatus', default_factory=list)

class KeySetData(ObjectModel):
    access: str = ApiField(alias='Access')
    algorithm: str = ApiField(alias='Algorithm')
    fingerprint_md5: str = ApiField(alias='FingerprintMD5')
    fingerprint_sha256: str = ApiField(alias='FingerprintSHA256')
    keyset_id: str = ApiField(alias='KeysetId')
    last_rotation_date: datetime = ApiField(alias='LastRotationDate')
    last_used: datetime = ApiField(alias='LastUsed')
    length: int = ApiField(alias='Length')
    private_keys: list[KeyData] = ApiField(alias='PrivateKeys', default_factory=list)
    process_error: str = ApiField(alias='ProcessError')
    process_status: str = ApiField(alias='ProcessStatus')
    public_keys: list[KeyData] = ApiField(alias='PublicKeys', default_factory=list)
    rotation_stage: int = ApiField(alias='RotationStage')
    type: str = ApiField(alias='Type')
    violation_status: list = ApiField(alias='ViolationStatus')

class KeyUsageData(ObjectModel):
    alert: int = ApiField(alias='Alert')
    authorized_key_id: int = ApiField(alias='AuthorizedKeyId')
    client_name: str = ApiField(alias='ClientName')
    fingerprint: str = ApiField(alias='Fingerprint')
    key_usage_id: int = ApiField(alias='KeyUsageId')
    keyset_id: str = ApiField(alias='KeysetId')
    last_used: datetime = ApiField(alias='LastUsed')
    private_key_id: int = ApiField(alias='PrivateKeyId')
    server_account: str = ApiField(alias='ServerAccount')
    server_name: str = ApiField(alias='ServerName')

class SshDeviceFilter(ObjectModel):
    device_names: list[str] = ApiField(alias='DeviceNames', default_factory=list)
    is_compliant: bool = ApiField(alias='IsCompliant')
    type: str = ApiField(alias='Type')

class LogData(ObjectModel):
    log_record: str = ApiField(alias='LogRecord')
    log_utc_epoch_date: int = ApiField(alias='LogUtcEpochDate')

class KeySetFilter(ObjectModel):
    algorithm: list[str] = ApiField(alias='Algorithm')
    authorized_key_comment: list[str] = ApiField(alias='AuthorizedKeyComment')
    device_guids: list[str] = ApiField(alias='DeviceGuids')
    fingerprints_md5: list[str] = ApiField(alias='FingerprintsMD5')
    fingerprints_sha256: list[str] = ApiField(alias='FingerprintsSHA256')
    last_used: datetime = ApiField(alias='LastUsed')
    length: list[int] = ApiField(alias='Length')
    max_key_length: int = ApiField(alias='MaxKeyLength')
    process_statuses: list[int] = ApiField(alias='ProcessStatuses')
    type: str = ApiField(alias='Type')
    usage_filter_type: str = ApiField(alias='UsageFilterType')
    usernames: list[str] = ApiField(alias='Usernames')
    violation_statuses: list[int] = ApiField(alias='ViolationStatuses')

class KeyUsageFilter(ObjectModel):
    client_name: list[str] = ApiField(alias='ClientName')
    server_account: list[str] = ApiField(alias='ServerAccount')
    server_name: list[str] = ApiField(alias='ServerName')
    alert: list[int] = ApiField(alias='Alert')
